@@ -1,4 +1,4 @@
-#! /usr/bin/env python3
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
 #===============================================================================
@@ -6,7 +6,7 @@
 #
 # Here are dictionaries of ...
 #  different modules for building KLayout (http://www.klayout.de/index.php)
-#  version 0.28.13 or later on different Apple Mac OSX platforms.
+#  version 0.29.0 or later on different Apple Mac OSX platforms.
 #
 # This file is imported by 'build4mac.py' script.
 #===============================================================================
@@ -72,14 +72,16 @@ Qts += [ 'Qt6MacPorts', 'Qt6Brew' ]
 #   install with 'sudo port install [qt5|qt5-qttools]'
 # [Key Type Name] = 'Qt5MacPorts'
 Qt5MacPorts = { 'qmake' : '/opt/local/libexec/qt5/bin/qmake',
-                'deploy': '/opt/local/libexec/qt5/bin/macdeployqt'
+                'deploy': '/opt/local/libexec/qt5/bin/macdeployqt',
+                'libdir': '/opt/local/libexec/qt5/lib'
               }
 
 # Qt5 from Homebrew (https://brew.sh/)
 #   install with 'brew install qt5'
 # [Key Type Name] = 'Qt5Brew'
 Qt5Brew = { 'qmake' : '%s/opt/qt@5/bin/qmake' % DefaultHomebrewRoot,
-            'deploy': '%s/opt/qt@5/bin/macdeployqt' % DefaultHomebrewRoot
+            'deploy': '%s/opt/qt@5/bin/macdeployqt' % DefaultHomebrewRoot,
+            'libdir': '%s/opt/qt@5/lib' % DefaultHomebrewRoot
           }
 
 # Qt5 bundled with anaconda3 installed under /Applications/anaconda3/
@@ -87,7 +89,8 @@ Qt5Brew = { 'qmake' : '%s/opt/qt@5/bin/qmake' % DefaultHomebrewRoot,
 # If so, you need to make a symbolic link: /Applications/anaconda3 ---> $HOME/opt/anaconda3/
 # [Key Type Name] = 'Qt5Ana3'
 Qt5Ana3 = { 'qmake' : '/Applications/anaconda3/bin/qmake',
-            'deploy': '/Applications/anaconda3/bin/macdeployqt'
+            'deploy': '/Applications/anaconda3/bin/macdeployqt',
+            'libdir': '/Applications/anaconda3/lib'
           }
 
 #-------------------------------------------------------------------------
@@ -97,16 +100,26 @@ Qt5Ana3 = { 'qmake' : '/Applications/anaconda3/bin/qmake',
 #   install with 'sudo port install [qt6|qt6-qttools]'
 # [Key Type Name] = 'Qt6MacPorts'
 Qt6MacPorts = { 'qmake' : '/opt/local/libexec/qt6/bin/qmake',
-                'deploy': '/opt/local/libexec/qt6/bin/macdeployqt'
+                'deploy': '/opt/local/libexec/qt6/bin/macdeployqt',
+                'libdir': '/opt/local/libexec/qt6/lib'
               }
 
 # Qt6 from Homebrew (https://brew.sh/)
 #   install with 'brew install qt6'
 # [Key Type Name] = 'Qt6Brew'
 Qt6Brew = { 'qmake' : '%s/opt/qt@6/bin/qmake' % DefaultHomebrewRoot,
-            'deploy': '%s/opt/qt@6/bin/macdeployqt' % DefaultHomebrewRoot
+            'deploy': '%s/opt/qt@6/bin/macdeployqt' % DefaultHomebrewRoot,
+            'libdir': '%s/opt/qt@6/lib' % DefaultHomebrewRoot
           }
 
+# Consolidated dictionary kit for Qt[5|6]
+Qt56Dictionary  = { 'Qt5MacPorts': Qt5MacPorts,
+                    'Qt5Brew'    : Qt5Brew,
+                    'Qt5Ana3'    : Qt5Ana3,
+                    'Qt6MacPorts': Qt6MacPorts,
+                    'Qt6Brew'    : Qt6Brew
+                  }
+
 #-----------------------------------------------------
 # [2] Ruby
 #     * Dropped the followings (2023-10-24).
@@ -117,54 +130,69 @@ Qt6Brew = { 'qmake' : '%s/opt/qt@6/bin/qmake' % DefaultHomebrewRoot,
 #-----------------------------------------------------
 RubyNil  = [ 'nil' ]
 RubySys  = [ 'RubyMonterey', 'RubyVentura', 'RubySonoma' ]
-RubyExt  = [ 'Ruby32MacPorts', 'Ruby32Brew', 'RubyAnaconda3' ]
+RubyExt  = [ 'Ruby33MacPorts', 'Ruby33Brew', 'RubyAnaconda3' ]
 Rubies   = RubyNil + RubySys + RubyExt
 
 #-----------------------------------------------------
 # Whereabouts of different components of Ruby
 #-----------------------------------------------------
+# % which ruby
+#   /System/Library/Frameworks/Ruby.framework/Versions/2.6/usr/bin/ruby
+#
+# % ruby -v
+#   ruby 2.6.10p210 (2022-04-12 revision 67958) [universal.x86_64-darwin21]
+#
+# Where is the 'ruby.h' used to build the 'ruby' executable?
+#
+# % ruby -e "puts File.expand_path('ruby.h', RbConfig::CONFIG['rubyhdrdir'])"
+#   ===> /Applications/Xcode.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX13.1.sdk \
+#        /System/Library/Frameworks/Ruby.framework/Versions/2.6/usr/include/ruby-2.6.0/ruby.h
+#
 # Bundled with Monterey (12.x)
 # [Key Type Name] = 'Sys'
-MontereySDK     = "/Applications/Xcode.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX.sdk"
+MontereyXcSDK   = "/Applications/Xcode.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX.sdk"
+MontereyCLTSDK  = "/Library/Developer/CommandLineTools/SDKs/MacOSX.sdk"
 RubyMonterey    = { 'exe':  '/System/Library/Frameworks/Ruby.framework/Versions/2.6/usr/bin/ruby',
-                    'inc':  '%s/System/Library/Frameworks/Ruby.framework/Headers' % MontereySDK,
-                    'inc2': '%s/System/Library/Frameworks/Ruby.framework/Headers/ruby' % MontereySDK,
-                    'lib':  '%s/System/Library/Frameworks/Ruby.framework/Versions/2.6/usr/lib/libruby.tbd' % MontereySDK
+                    'inc':  '%s/System/Library/Frameworks/Ruby.framework/Headers' % MontereyXcSDK,
+                    'inc2': '%s/System/Library/Frameworks/Ruby.framework/Headers/ruby' % MontereyXcSDK,
+                    'lib':  '%s/System/Library/Frameworks/Ruby.framework/Versions/2.6/usr/lib/libruby.tbd' % MontereyXcSDK
                   }
 
 # Bundled with Ventura (13.x)
 # [Key Type Name] = 'Sys'
-VenturaSDK      = "/Applications/Xcode.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX.sdk"
+VenturaXcSDK    = "/Applications/Xcode.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX.sdk"
+VenturaCLTSDK   = "/Library/Developer/CommandLineTools/SDKs/MacOSX.sdk"
 RubyVentura     = { 'exe':  '/System/Library/Frameworks/Ruby.framework/Versions/2.6/usr/bin/ruby',
-                    'inc':  '%s/System/Library/Frameworks/Ruby.framework/Headers' % VenturaSDK,
-                    'inc2': '%s/System/Library/Frameworks/Ruby.framework/Headers/ruby' % VenturaSDK,
-                    'lib':  '%s/System/Library/Frameworks/Ruby.framework/Versions/2.6/usr/lib/libruby.tbd' % VenturaSDK
+                    'inc':  '%s/System/Library/Frameworks/Ruby.framework/Headers' % VenturaXcSDK,
+                    'inc2': '%s/System/Library/Frameworks/Ruby.framework/Headers/ruby' % VenturaXcSDK,
+                    'lib':  '%s/System/Library/Frameworks/Ruby.framework/Versions/2.6/usr/lib/libruby.tbd' % VenturaXcSDK
                   }
 
 # Bundled with Sonoma (14.x)
 # [Key Type Name] = 'Sys'
-SonomaSDK       = "/Applications/Xcode.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX.sdk"
+SonomaXcSDK     = "/Applications/Xcode.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX.sdk"
+SonomaCLTSDK    = "/Library/Developer/CommandLineTools/SDKs/MacOSX.sdk"
 RubySonoma      = { 'exe':  '/System/Library/Frameworks/Ruby.framework/Versions/2.6/usr/bin/ruby',
-                    'inc':  '%s/System/Library/Frameworks/Ruby.framework/Headers' % SonomaSDK,
-                    'inc2': '%s/System/Library/Frameworks/Ruby.framework/Headers/ruby' % SonomaSDK,
-                    'lib':  '%s/System/Library/Frameworks/Ruby.framework/Versions/2.6/usr/lib/libruby.tbd' % SonomaSDK
+                    'inc':  '%s/System/Library/Frameworks/Ruby.framework/Headers' % SonomaXcSDK,
+                    'inc2': '%s/System/Library/Frameworks/Ruby.framework/Headers/ruby' % SonomaXcSDK,
+                    'lib':  '%s/System/Library/Frameworks/Ruby.framework/Versions/2.6/usr/lib/libruby.tbd' % SonomaXcSDK
                   }
 
-# Ruby 3.2 from MacPorts (https://www.macports.org/)
-#  install with 'sudo port install ruby32'
-# [Key Type Name] = 'MP32'
-Ruby32MacPorts  = { 'exe': '/opt/local/bin/ruby3.2',
-                    'inc': '/opt/local/include/ruby-3.2.2',
-                    'lib': '/opt/local/lib/libruby.3.2.dylib'
+# Ruby 3.3 from MacPorts (https://www.macports.org/)
+#  install with 'sudo port install ruby33'
+# [Key Type Name] = 'MP33'
+Ruby33MacPorts  = { 'exe': '/opt/local/bin/ruby3.3',
+                    'inc': '/opt/local/include/ruby-3.3.0',
+                    'lib': '/opt/local/lib/libruby.3.3.dylib'
                   }
 
-# Ruby 3.2 from Homebrew
-#   install with 'brew install ruby@3.2'
-# [Key Type Name] = 'HB32'
-HBRuby32Path    = '%s/opt/ruby@3.2' % DefaultHomebrewRoot
-Ruby32Brew      = { 'exe': '%s/bin/ruby' % HBRuby32Path,
-                    'inc': '%s/include/ruby-3.2.0' % HBRuby32Path,
-                    'lib': '%s/lib/libruby.3.2.dylib' % HBRuby32Path
+# Ruby 3.3 from Homebrew
+#   install with 'brew install ruby@3.3'
+# [Key Type Name] = 'HB33'
+HBRuby33Path    = '%s/opt/ruby@3.3' % DefaultHomebrewRoot
+Ruby33Brew      = { 'exe': '%s/bin/ruby' % HBRuby33Path,
+                    'inc': '%s/include/ruby-3.3.0' % HBRuby33Path,
+                    'lib': '%s/lib/libruby.3.3.dylib' % HBRuby33Path
                   }
 
 # Ruby 3.2 bundled with anaconda3 installed under /Applications/anaconda3/
@@ -181,8 +209,8 @@ RubyDictionary  = { 'nil'           : None,
                     'RubyMonterey'  : RubyMonterey,
                     'RubyVentura'   : RubyVentura,
                     'RubySonoma'    : RubySonoma,
-                    'Ruby32MacPorts': Ruby32MacPorts,
-                    'Ruby32Brew'    : Ruby32Brew,
+                    'Ruby33MacPorts': Ruby33MacPorts,
+                    'Ruby33Brew'    : Ruby33Brew,
                     'RubyAnaconda3' : RubyAnaconda3
                   }
 
@@ -195,7 +223,7 @@ RubyDictionary  = { 'nil'           : None,
 #           for the previous states.
 #-----------------------------------------------------
 PythonNil  = [ 'nil' ]
-PythonSys  = [ ]
+PythonSys  = [ 'PythonMonterey', 'PythonVentura', 'PythonSonoma' ]
 PythonExt  = [ 'Python39MacPorts', 'Python39Brew' ]
 PythonExt += [ 'Python311MacPorts', 'Python311Brew' ]
 PythonExt += [ 'PythonAnaconda3', 'PythonAutoBrew' ]
@@ -204,6 +232,33 @@ Pythons    = PythonNil + PythonSys + PythonExt
 #-----------------------------------------------------
 # Whereabouts of different components of Python
 #-----------------------------------------------------
+# Bundled with Monterey (12.x)
+# [Key Type Name] = 'Sys'
+MontereyPy3FWXc = "/Applications/Xcode.app/Contents/Developer/Library/Frameworks"
+MontereyPy3FW   = "/Library/Developer/CommandLineTools/Library/Frameworks"
+PythonMonterey  = { 'exe': '%s/Python3.framework/Versions/3.9/bin/python3.9' % MontereyPy3FW,
+                    'inc': '%s/Python3.framework/Versions/3.9/include/python3.9' % MontereyPy3FW,
+                    'lib': '%s/Python3.framework/Versions/3.9/lib/libpython3.9.dylib' % MontereyPy3FW
+                  }
+
+# Bundled with Ventura (13.x)
+# [Key Type Name] = 'Sys'
+VenturaPy3FWXc  = "/Applications/Xcode.app/Contents/Developer/Library/Frameworks"
+VenturaPy3FW    = "/Library/Developer/CommandLineTools/Library/Frameworks"
+PythonVentura   = { 'exe': '%s/Python3.framework/Versions/3.9/bin/python3.9' % VenturaPy3FW,
+                    'inc': '%s/Python3.framework/Versions/3.9/include/python3.9' % VenturaPy3FW,
+                    'lib': '%s/Python3.framework/Versions/3.9/lib/libpython3.9.dylib' % VenturaPy3FW
+                  }
+
+# Bundled with Sonoma (14.x)
+# [Key Type Name] = 'Sys'
+SonomaPy3FWXc   = "/Applications/Xcode.app/Contents/Developer/Library/Frameworks"
+SonomaPy3FW     = "/Library/Developer/CommandLineTools/Library/Frameworks"
+PythonSonoma    = { 'exe': '%s/Python3.framework/Versions/3.9/bin/python3.9' % SonomaPy3FW,
+                    'inc': '%s/Python3.framework/Versions/3.9/include/python3.9' % SonomaPy3FW,
+                    'lib': '%s/Python3.framework/Versions/3.9/lib/libpython3.9.dylib' % SonomaPy3FW
+                  }
+
 # Python 3.9 from MacPorts (https://www.macports.org/)
 #   install with 'sudo port install python39'
 # [Key Type Name] = 'MP39'
@@ -299,6 +354,9 @@ else:
 
 # Consolidated dictionary kit for Python
 PythonDictionary = { 'nil'              : None,
+                     'PythonMonterey'   : PythonMonterey,
+                     'PythonVentura'    : PythonVentura,
+                     'PythonSonoma'     : PythonSonoma,
                      'Python39MacPorts' : Python39MacPorts,
                      'Python311MacPorts': Python311MacPorts,
                      'Python39Brew'     : Python39Brew,
@@ -312,7 +370,7 @@ if _have_Homebrew_Python:
 # [4] KLayout executables including buddy tools
 #-----------------------------------------------------
 KLayoutExecs  = [ 'klayout' ]
-KLayoutExecs += [ 'strm2cif', 'strm2dxf', 'strm2gds', 'strm2gdstxt', 'strm2oas' ]
+KLayoutExecs += [ 'strm2cif', 'strm2dxf', 'strm2gds', 'strm2gdstxt', 'strm2mag', 'strm2oas' ]
 KLayoutExecs += [ 'strm2txt', 'strmclip', 'strmcmp',  'strmrun',     'strmxor'  ]
 
 #----------------
