@@ -4,7 +4,7 @@
 # File: "macbuild/macQAT.sh"
 #
 #  The top Bash script to run "ut_runner" after building KLayout
-#  (http://www.klayout.de/index.php) version 0.26.1 or later on different Apple
+#  (http://www.klayout.de/index.php) version 0.25.17 or later on different Apple
 #  Mac OSX platforms.
 #
 #  This script must be copied to a "*.macQAT/" directory to run.
@@ -46,7 +46,7 @@ logfile=QATest_${gitSHA1}_${timestamp}__${presentDir}.log
 #----------------------------------------------------------------
 export TESTSRC=..
 export TESTTMP=QATest_${gitSHA1}_${timestamp}__${presentDir}
-export DYLD_LIBRARY_PATH=$(pwd):$(pwd)/db_plugins:$(pwd)/lay_plugins
+export DYLD_LIBRARY_PATH=$(pwd):$(pwd)/db_plugins:$(pwd)/lay_plugins:$(pwd)/pymod
 
 #----------------------------------------------------------------
 # Environment variables for "ut_runner"
