@@ -431,7 +431,7 @@ public:
     // .. nothing yet ..
   }
 
-  void execute (const tl::ExpressionParserContext &context, tl::Variant &out, const std::vector <tl::Variant> &vv) const
+  void execute (const tl::ExpressionParserContext &context, tl::Variant &out, const std::vector <tl::Variant> &vv, const std::map<std::string, tl::Variant> * /*kwargs*/) const
   {
     if (vv.size () != 0) {
       throw tl::EvalError (tl::to_string (tr ("Annotation function must not have arguments")), context);
