@@ -53,8 +53,14 @@ public:
 
   virtual EdgePairsDelegate *filtered (const EdgePairFilterBase &) const;
 
-  virtual RegionDelegate *processed_to_polygons (const EdgePairToPolygonProcessorBase &filter) const;
-  virtual EdgesDelegate *processed_to_edges (const EdgePairToEdgeProcessorBase &filter) const;
+  virtual EdgePairsDelegate *process_in_place (const EdgePairProcessorBase &proc)
+  {
+    return processed (proc);
+  }
+
+  virtual EdgePairsDelegate *processed (const EdgePairProcessorBase &proc) const;
+  virtual RegionDelegate *processed_to_polygons (const EdgePairToPolygonProcessorBase &proc) const;
+  virtual EdgesDelegate *processed_to_edges (const EdgePairToEdgeProcessorBase &proc) const;
 
   virtual EdgePairsDelegate *add_in_place (const EdgePairs &other)
   {
