@@ -1079,7 +1079,7 @@ public:
         m_cl.erase (cli);
       }
 
-    } else if (m_report_single) {
+    } else if (m_report_single && m_ignore_single.find (obj) == m_ignore_single.end ()) {
 
       //  single-object entry: create a cluster and feed it a single-object signature
       Cluster cl (m_cl_template);
@@ -1089,6 +1089,13 @@ public:
     }
   }
 
+  void ignore_single (const Obj *o)
+  {
+    if (m_report_single) {
+      m_ignore_single.insert (o);
+    }
+  }
+
   void add_asymm (const Obj *o1, const Prop &p1, const Obj *o2, const Prop &p2)
   {
     om_iterator_type om1 = m_om.find (om_key_type (o1, p1));
@@ -1166,6 +1173,7 @@ private:
   bool m_report_single;
   cl_type m_cl;
   om_type m_om;
+  std::set<const Obj *> m_ignore_single;
 };
 
 }
