@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -182,6 +182,13 @@ public:
    */
   void copy (unsigned int src, unsigned int dest);
 
+  /**
+   *  @brief Copy the shapes from layer src to dest (only shapes from given classes)
+   *
+   *  The target layer is not overwritten. Instead, the shapes are added to the target layer's shapes.
+   */
+  void copy (unsigned int src, unsigned int dest, unsigned int types);
+
   /**
    *  @brief Move the shapes from layer src to dest
    *
@@ -189,6 +196,13 @@ public:
    */
   void move (unsigned int src, unsigned int dest);
 
+  /**
+   *  @brief Move the shapes from layer src to dest (only shapes from given classes)
+   *
+   *  The target layer is not overwritten. Instead, the shapes are added to the target layer's shapes.
+   */
+  void move (unsigned int src, unsigned int dest, unsigned int types);
+
   /**
    *  @brief Swap the layers given
    */
@@ -199,7 +213,12 @@ public:
    */
   void clear (unsigned int index);
 
-  /** 
+  /**
+   *  @brief Clear the shapes on the given layer (only the shapes from the given classes)
+   */
+  void clear (unsigned int index, unsigned int types);
+
+  /**
    *  @brief Erase a cell instance given by a instance proxy
    *
    *  Erasing a cell instance will destroy the sorting order and invalidate
@@ -1057,7 +1076,7 @@ private:
   db::properties_id_type m_prop_id;
 
   // packed fields
-  unsigned int m_hier_levels : 29;
+  unsigned int m_hier_levels : 30;
   bool m_bbox_needs_update : 1;
   bool m_ghost_cell : 1;
 
@@ -1128,9 +1147,9 @@ private:
    *  This will sort the cell instance list. As a prerequesite
    *  the cell's bounding boxes must have been computed.
    *
-   *  @param layers The maximum number of layers in the child cells
+   *  @param force Force sorting, even if not strictly needed
    */
-  void sort_inst_tree ();
+  void sort_inst_tree (bool force);
 };
 
 /**
