@@ -759,6 +759,11 @@ public:
    */
   void blank ();
 
+  /**
+   *  @brief Gets a value indicating whether the circuit is empty
+   */
+  bool is_empty () const;
+
   /**
    *  @brief Generate memory statistics
    */
