@@ -797,6 +797,10 @@ DeepRegion::and_with (const Region &other, PropertyConstraint property_constrain
 
     return AsIfFlatRegion::and_with (other, property_constraint);
 
+  } else if (other_deep->deep_layer () == deep_layer () && pc_skip (property_constraint)) {
+
+    return clone ();
+
   } else {
 
     return new DeepRegion (and_or_not_with (other_deep, true, property_constraint));
@@ -817,6 +821,10 @@ DeepRegion::not_with (const Region &other, PropertyConstraint property_constrain
 
     return AsIfFlatRegion::not_with (other, property_constraint);
 
+  } else if (other_deep->deep_layer () == deep_layer () && pc_skip (property_constraint)) {
+
+    return new DeepRegion (deep_layer ().derived ());
+
   } else {
 
     return new DeepRegion (and_or_not_with (other_deep, false, property_constraint));
@@ -825,8 +833,13 @@ DeepRegion::not_with (const Region &other, PropertyConstraint property_constrain
 }
 
 RegionDelegate *
-DeepRegion::or_with (const Region &other, db::PropertyConstraint /*property_constraint*/) const
+DeepRegion::or_with (const Region &other, db::PropertyConstraint property_constraint) const
 {
+  const DeepRegion *other_deep = dynamic_cast <const DeepRegion *> (other.delegate ());
+  if (other_deep && other_deep->deep_layer () == deep_layer () && pc_skip (property_constraint)) {
+    return clone ();
+  }
+
   //  TODO: implement property_constraint
   RegionDelegate *res = add (other);
   return res->merged_in_place ();
@@ -849,6 +862,10 @@ DeepRegion::andnot_with (const Region &other, PropertyConstraint property_constr
 
     return AsIfFlatRegion::andnot_with (other, property_constraint);
 
+  } else if (other_deep->deep_layer () == deep_layer () && pc_skip (property_constraint)) {
+
+    return std::make_pair (clone (), new DeepRegion (deep_layer ().derived ()));
+
   } else {
 
     std::pair<DeepLayer, DeepLayer> res = and_and_not_with (other_deep, property_constraint);
@@ -962,6 +979,10 @@ DeepRegion::xor_with (const Region &other, db::PropertyConstraint property_const
 
     return AsIfFlatRegion::xor_with (other, property_constraint);
 
+  } else if (other_deep->deep_layer () == deep_layer () && pc_skip (property_constraint)) {
+
+    return new DeepRegion (deep_layer ().derived ());
+
   } else {
 
     //  Implement XOR as (A-B)+(B-A) - only this implementation
@@ -1335,7 +1356,7 @@ DeepRegion::snapped (db::Coord gx, db::Coord gy)
 }
 
 EdgesDelegate *
-DeepRegion::edges (const EdgeFilterBase *filter) const
+DeepRegion::edges (const EdgeFilterBase *filter, const PolygonToEdgeProcessorBase *proc) const
 {
   std::unique_ptr<db::DeepEdges> res (new db::DeepEdges (deep_layer ().derived ()));
 
@@ -1343,7 +1364,7 @@ DeepRegion::edges (const EdgeFilterBase *filter) const
     return res.release ();
   }
 
-  if (! filter && merged_semantics () && ! merged_polygons_available ()) {
+  if (! proc && ! filter && merged_semantics () && ! merged_polygons_available ()) {
 
     //  Hierarchical edge detector - no pre-merge required
 
@@ -1388,15 +1409,32 @@ DeepRegion::edges (const EdgeFilterBase *filter) const
       const db::Shapes &s = c->shapes (polygons.layer ());
       db::Shapes &st = c->shapes (res->deep_layer ().layer ());
 
+      std::vector<db::Edge> heap;
+
       for (db::Shapes::shape_iterator si = s.begin (db::ShapeIterator::All); ! si.at_end (); ++si) {
 
         db::Polygon poly;
         si->polygon (poly);
 
-        for (db::Polygon::polygon_edge_iterator e = poly.begin_edge (); ! e.at_end (); ++e) {
-          if (! filter || filter->selected ((*e).transformed (tr))) {
-            st.insert (db::EdgeWithProperties (*e, pm (si->prop_id ())));
+        if (proc) {
+
+          heap.clear ();
+          proc->process (poly, heap);
+
+          for (auto e = heap.begin (); e != heap.end (); ++e) {
+            if (! filter || filter->selected ((*e).transformed (tr))) {
+              st.insert (db::EdgeWithProperties (*e, pm (si->prop_id ())));
+            }
           }
+
+        } else {
+
+          for (db::Polygon::polygon_edge_iterator e = poly.begin_edge (); ! e.at_end (); ++e) {
+            if (! filter || filter->selected ((*e).transformed (tr))) {
+              st.insert (db::EdgeWithProperties (*e, pm (si->prop_id ())));
+            }
+          }
+
         }
 
       }
@@ -2110,6 +2148,16 @@ DeepRegion::in_and_out_generic (const Region &other, InteractingOutputMode outpu
     other_deep = dr_holder.get ();
   }
 
+  if (deep_layer () == other_deep->deep_layer ()) {
+    if (output_mode == PositiveAndNegative) {
+      return std::make_pair (clone (), new DeepRegion (deep_layer ().derived ()));
+    } else if (output_mode == Negative) {
+      return std::make_pair (new DeepRegion (deep_layer ().derived ()), (RegionDelegate *) 0);
+    } else {
+      return std::make_pair (clone (), (RegionDelegate *) 0);
+    }
+  }
+
   const db::DeepLayer &polygons = merged_deep_layer ();
   const db::DeepLayer &other_polygons = other_deep->merged_deep_layer ();
 
@@ -2171,6 +2219,26 @@ DeepRegion::selected_interacting_generic (const Region &other, int mode, bool to
     other_deep = dr_holder.get ();
   }
 
+  if (deep_layer () == other_deep->deep_layer () && !counting) {
+    if (mode <= 0 /*inside or interacting*/) {
+      if (output_mode == PositiveAndNegative) {
+        return std::make_pair (clone (), new DeepRegion (deep_layer ().derived ()));
+      } else if (output_mode == Negative) {
+        return std::make_pair (new DeepRegion (deep_layer ().derived ()), (RegionDelegate *) 0);
+      } else {
+        return std::make_pair (clone (), (RegionDelegate *) 0);
+      }
+    } else {
+      if (output_mode == PositiveAndNegative) {
+        return std::make_pair (new DeepRegion (deep_layer ().derived ()), clone ());
+      } else if (output_mode == Negative) {
+        return std::make_pair (clone (), (RegionDelegate *) 0);
+      } else {
+        return std::make_pair (new DeepRegion (deep_layer ().derived ()), (RegionDelegate *) 0);
+      }
+    }
+  }
+
   const db::DeepLayer &polygons = merged_deep_layer ();
   //  NOTE: with counting, the other polygons must be merged
   const db::DeepLayer &other_polygons = counting ? other_deep->merged_deep_layer () : other_deep->deep_layer ();
@@ -2188,7 +2256,13 @@ DeepRegion::selected_interacting_generic (const Region &other, int mode, bool to
   bool result_is_merged = (! split_after && (merged_semantics () || is_merged ()));
   InteractingResultHolder orh (output_mode, result_is_merged, polygons);
 
-  proc.run (&op, polygons.layer (), other_polygons.layer (), orh.layers ());
+  if (polygons == other_polygons) {
+    //  with counting and two identical inputs we need to create a layer copy
+    db::DeepLayer other_copy (other_polygons.copy ());
+    proc.run (&op, polygons.layer (), other_copy.layer (), orh.layers ());
+  } else {
+    proc.run (&op, polygons.layer (), other_polygons.layer (), orh.layers ());
+  }
 
   return orh.result_pair ();
 }
@@ -2268,6 +2342,10 @@ DeepRegion::pull_generic (const Region &other, int mode, bool touching) const
     other_deep = dr_holder.get ();
   }
 
+  if (deep_layer () == other_deep->deep_layer ()) {
+    return clone ();
+  }
+
   //  in "inside" mode, the first argument needs to be merged too
   const db::DeepLayer &polygons = mode < 0 ? merged_deep_layer () : deep_layer ();
   const db::DeepLayer &other_polygons = other_deep->merged_deep_layer ();
