@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -49,6 +49,7 @@ class Texts;
 class ShapeCollection;
 class NetBuilder;
 class LayoutToNetlist;
+class VariantsCollectorBase;
 
 /**
  *  @brief Represents a shape collection from the deep shape store
@@ -164,6 +165,11 @@ public:
    */
   const std::set<db::cell_index_type> *breakout_cells () const;
 
+  /**
+   *  @brief Gets a hash value representing the breakout cells
+   */
+  size_t breakout_cells_hash () const;
+
   /**
    *  @brief Inserts the layer into the given layout, starting from the given cell and into the given layer
    */
@@ -192,25 +198,6 @@ public:
    */
   void add_from (const DeepLayer &dl);
 
-  /**
-   *  @brief Separates cell variants (see DeepShapeStore::separate_variants)
-   */
-  template <class VarCollector>
-  void separate_variants (VarCollector &collector);
-
-  /**
-   *  @brief Commits shapes for variants to the existing cell hierarchy
-   *
-   *  The "to_propagate" collection is a set of shapes per cell and variant. The
-   *  algorithm will put these shapes into the existing hierarchy putting the
-   *  shapes into the proper parent cells to resolve variants.
-   *
-   *  This map will be modified by the algorithm and should be discarded
-   *  later.
-   */
-  template <class VarCollector>
-  void commit_shapes (VarCollector &collector, std::map<db::cell_index_type, std::map<db::ICplxTrans, db::Shapes> > &to_propagate);
-
   /**
    *  @brief Gets the shape store object
    *  This is a pure const version to prevent manipulation of the store.
@@ -269,6 +256,7 @@ public:
   bool reject_odd_polygons () const;
 
   const std::set<db::cell_index_type> *breakout_cells (unsigned int layout_index) const;
+  size_t breakout_cells_hash (unsigned int layout_index) const;
   void clear_breakout_cells (unsigned int layout_index);
   void set_breakout_cells (unsigned int layout_index, const std::set<db::cell_index_type> &boc);
   void add_breakout_cell (unsigned int layout_index, db::cell_index_type ci);
@@ -283,14 +271,14 @@ private:
   size_t m_max_vertex_count;
   bool m_reject_odd_polygons;
   tl::Variant m_text_property_name;
-  std::vector<std::set<db::cell_index_type> > m_breakout_cells;
+  std::vector<std::pair<std::set<db::cell_index_type>, size_t> > m_breakout_cells;
   int m_text_enlargement;
   bool m_subcircuit_hierarchy_for_nets;
 
-  std::set<db::cell_index_type> &ensure_breakout_cells (unsigned int layout_index)
+  std::pair<std::set<db::cell_index_type>, size_t> &ensure_breakout_cells (unsigned int layout_index)
   {
     if (m_breakout_cells.size () <= size_t (layout_index)) {
-      m_breakout_cells.resize (layout_index + 1, std::set<db::cell_index_type> ());
+      m_breakout_cells.resize (layout_index + 1, std::pair<std::set<db::cell_index_type>, size_t> ());
     }
     return m_breakout_cells [layout_index];
   }
@@ -298,7 +286,7 @@ private:
 
 struct DB_PUBLIC RecursiveShapeIteratorCompareForTargetHierarchy
 {
-  bool operator () (const std::pair<db::RecursiveShapeIterator, db::ICplxTrans> &a, const std::pair<db::RecursiveShapeIterator, db::ICplxTrans> &b) const
+  bool operator () (const std::pair<db::RecursiveShapeIterator, std::pair<size_t, db::ICplxTrans> > &a, const std::pair<db::RecursiveShapeIterator, std::pair<size_t, db::ICplxTrans> > &b) const
   {
     int cmp_iter = db::compare_iterators_with_respect_to_target_hierarchy (a.first, b.first);
     if (cmp_iter != 0) {
@@ -555,42 +543,6 @@ public:
    */
   const db::CellMapping &internal_cell_mapping (unsigned int from_layout_index, unsigned int into_layout_index);
 
-  /**
-   *  @brief Create cell variants from the given variant collector
-   *
-   *  To use this method, first create a variant collector (db::cell_variant_collector) with the required
-   *  reducer and collect the variants. Then call this method on the desired layout index to create the variants.
-   */
-  template <class VarCollector>
-  void separate_variants (unsigned int layout_index, VarCollector &coll)
-  {
-    tl_assert (is_valid_layout_index (layout_index));
-
-    std::map<db::cell_index_type, std::map<db::ICplxTrans, db::cell_index_type> > var_map;
-    coll.separate_variants (layout (layout_index), initial_cell (layout_index), &var_map);
-    if (var_map.empty ()) {
-      //  nothing to do.
-      return;
-    }
-
-    issue_variants (layout_index, var_map);
-  }
-
-  /**
-   *  @brief Commits shapes for variants to the existing cell hierarchy
-   *
-   *  To use this method, first create a variant collector (db::cell_variant_collector) with the required
-   *  reducer and collect the variants. Then call this method on the desired layout index to commit the shapes for the
-   *  respective variants.
-   */
-  template <class VarCollector>
-  void commit_shapes (unsigned int layout_index, VarCollector &coll, unsigned int layer, std::map<db::cell_index_type, std::map<db::ICplxTrans, db::Shapes> > &to_commit)
-  {
-    tl_assert (is_valid_layout_index (layout_index));
-
-    coll.commit_shapes (layout (layout_index), initial_cell (layout_index), layer, to_commit);
-  }
-
   /**
    *  @brief For testing
    */
@@ -727,6 +679,23 @@ public:
    */
   int threads () const;
 
+  /**
+   *  @brief Sets a flag indicating whether the working layouts will store the whole original hierarchy
+   *
+   *  Setting this flag to true will make the deep shape store copy the
+   *  hierarchy exactly from the origin layouts. This will take somewhat
+   *  more memory but avoid future cell hierarchy mapping operations.
+   *
+   *  If set to false, only the needed parts of the hierarchy are copied.
+   *  This part may need to grow when further operations are triggered.
+   */
+  void set_wants_all_cells (bool f);
+
+  /**
+   *  @brief Gets a flag indicating whether the working layouts will store the whole original hierarchy
+   */
+  bool wants_all_cells () const;
+
   /**
    *  @brief Sets a flag indicating whether to reject odd polygons
    *
@@ -819,6 +788,11 @@ public:
    */
   const std::set<db::cell_index_type> *breakout_cells (unsigned int layout_index) const;
 
+  /**
+   *  @brief Gets a hash value representing the breakout cells
+   */
+  size_t breakout_cells_hash (unsigned int layout_index) const;
+
   /**
    *  @brief Clears the breakout cell list for a given layout
    */
@@ -865,7 +839,7 @@ private:
 
   void issue_variants (unsigned int layout, const std::map<db::cell_index_type, std::map<db::ICplxTrans, db::cell_index_type> > &var_map);
 
-  typedef std::map<std::pair<db::RecursiveShapeIterator, db::ICplxTrans>, unsigned int, RecursiveShapeIteratorCompareForTargetHierarchy> layout_map_type;
+  typedef std::map<std::pair<db::RecursiveShapeIterator, std::pair<size_t, db::ICplxTrans> >, unsigned int, RecursiveShapeIteratorCompareForTargetHierarchy> layout_map_type;
 
   //  no copying
   DeepShapeStore (const DeepShapeStore &);
@@ -878,6 +852,7 @@ private:
   DeepShapeStoreState m_state;
   std::list<DeepShapeStoreState> m_state_stack;
   bool m_keep_layouts;
+  bool m_wants_all_cells;
   tl::Mutex m_lock;
 
   struct DeliveryMappingCacheKey
@@ -910,20 +885,6 @@ private:
   std::map<std::pair<unsigned int, unsigned int>, CellMappingWithGenerationIds> m_internal_mapping_cache;
 };
 
-template <class VarCollector>
-void DeepLayer::separate_variants (VarCollector &collector)
-{
-  check_dss ();
-  mp_store->separate_variants (m_layout, collector);
-}
-
-template <class VarCollector>
-void DeepLayer::commit_shapes (VarCollector &collector, std::map<db::cell_index_type, std::map<db::ICplxTrans, db::Shapes> > &to_commit)
-{
-  check_dss ();
-  mp_store->commit_shapes (m_layout, collector, layer (), to_commit);
-}
-
 }
 
 #endif
