@@ -80,6 +80,8 @@ public:
   virtual TextsDelegate *filter_in_place (const TextFilterBase &filter);
   virtual TextsDelegate *filtered (const TextFilterBase &) const;
 
+  virtual TextsDelegate *process_in_place (const TextProcessorBase &);
+  virtual TextsDelegate *processed (const TextProcessorBase &) const;
   virtual RegionDelegate *processed_to_polygons (const TextToPolygonProcessorBase &filter) const;
 
   virtual TextsDelegate *add_in_place (const Texts &other);
