@@ -251,6 +251,13 @@ struct CutPoints
 
     } 
 
+    //  do not insert points twice
+    for (auto c = cut_points.begin (); c != cut_points.end (); ++c) {
+      if (*c == p) {
+        return;
+      }
+    }
+
     cut_points.push_back (p);
 
   }
@@ -1057,6 +1064,12 @@ EdgeProcessor::reserve (size_t n)
   mp_work_edges->reserve (n);
 }
 
+size_t
+EdgeProcessor::count () const
+{
+  return mp_work_edges->size ();
+}
+
 void 
 EdgeProcessor::insert (const db::Edge &e, EdgeProcessor::property_type p)
 {
