@@ -695,6 +695,11 @@ public:
    */
   void reserve (size_t n);
 
+  /**
+   *  @brief Reports the number of edges stored in the processor
+   */
+  size_t count () const;
+
   /**
    *  @brief Insert an edge 
    */
