@@ -255,12 +255,12 @@ public:
   virtual std::pair<EdgesDelegate *, EdgesDelegate *> inside_outside_part_pair (const Region &other) const = 0;
   virtual RegionDelegate *pull_interacting (const Region &) const = 0;
   virtual EdgesDelegate *pull_interacting (const Edges &) const = 0;
-  virtual EdgesDelegate *selected_interacting (const Region &other) const = 0;
-  virtual EdgesDelegate *selected_not_interacting (const Region &other) const = 0;
-  virtual EdgesDelegate *selected_interacting (const Edges &other) const = 0;
-  virtual EdgesDelegate *selected_not_interacting (const Edges &other) const = 0;
-  virtual std::pair<EdgesDelegate *, EdgesDelegate *> selected_interacting_pair (const Region &other) const = 0;
-  virtual std::pair<EdgesDelegate *, EdgesDelegate *> selected_interacting_pair (const Edges &other) const = 0;
+  virtual EdgesDelegate *selected_interacting (const Region &other, size_t min_count, size_t max_count) const = 0;
+  virtual EdgesDelegate *selected_not_interacting (const Region &other, size_t min_count, size_t max_count) const = 0;
+  virtual EdgesDelegate *selected_interacting (const Edges &other, size_t min_count, size_t max_count) const = 0;
+  virtual EdgesDelegate *selected_not_interacting (const Edges &other, size_t min_count, size_t max_count) const = 0;
+  virtual std::pair<EdgesDelegate *, EdgesDelegate *> selected_interacting_pair (const Region &other, size_t min_count, size_t max_count) const = 0;
+  virtual std::pair<EdgesDelegate *, EdgesDelegate *> selected_interacting_pair (const Edges &other, size_t min_count, size_t max_count) const = 0;
 
   virtual EdgesDelegate *selected_outside (const Region &other) const = 0;
   virtual EdgesDelegate *selected_not_outside (const Region &other) const = 0;
