@@ -142,22 +142,23 @@ private:
 class DB_PUBLIC EdgeAngleChecker
 {
 public:
-  EdgeAngleChecker (double angle_start, bool include_angle_start, double angle_end, bool include_angle_end);
+  EdgeAngleChecker (double angle_start, bool include_angle_start, double angle_end, bool include_angle_end, bool inverse, bool absolute);
 
   bool operator() (const db::Edge &a, const db::Edge &b) const
   {
-    return m_all || check (a.d (), b.d ());
+    return (m_all || check (a.d (), b.d ()) || (m_absolute && check (b.d (), a.d ()))) != m_inverse;
   }
 
   bool operator() (const db::Vector &a, const db::Vector &b) const
   {
-    return m_all || check (a, b);
+    return (m_all || check (a, b) || (m_absolute && check (b, a))) != m_inverse;
   }
 
 private:
   db::CplxTrans m_t_start, m_t_end;
   bool m_include_start, m_include_end;
   bool m_big_angle, m_all;
+  bool m_inverse, m_absolute;
 
   bool check (const db::Vector &a, const db::Vector &b) const;
 };
@@ -181,22 +182,24 @@ struct DB_PUBLIC EdgeOrientationFilter
    *  @param amin The minimum angle (measured against the x axis)
    *  @param amax The maximum angle (measured against the x axis)
    *  @param inverse If set to true, only edges not matching this criterion will be filtered
+   *  @param absolute Angles are always positive
    *
    *  This filter will filter out all edges whose angle against x axis
    *  is larger or equal to amin and less than amax.
    */
-  EdgeOrientationFilter (double amin, bool include_amin, double amax, bool include_amax, bool inverse);
+  EdgeOrientationFilter (double amin, bool include_amin, double amax, bool include_amax, bool inverse, bool absolute);
 
   /**
    *  @brief Constructor
    *
    *  @param a The angle (measured against the x axis)
    *  @param inverse If set to true, only edges not matching this criterion will be filtered
+   *  @param absolute Angles are always positive
    *
    *  This filter will filter out all edges whose angle against x axis
    *  is equal to a.
    */
-  EdgeOrientationFilter (double a, bool inverse);
+  EdgeOrientationFilter (double a, bool inverse, bool absolute);
 
   /**
    *  @brief Returns true if the edge orientation matches the criterion
@@ -339,16 +342,39 @@ class edge_interaction_filter
   : public db::box_scanner_receiver<db::Edge, size_t>
 {
 public:
-  edge_interaction_filter (OutputContainer &output, EdgeInteractionMode mode)
-    : mp_output (&output), m_mode (mode)
+  edge_interaction_filter (OutputContainer &output, EdgeInteractionMode mode, size_t min_count, size_t max_count)
+    : mp_output (&output), m_mode (mode), m_min_count (min_count), m_max_count (max_count)
   {
-    //  .. nothing yet ..
+    //  NOTE: "counting" does not really make much sense in Outside mode ...
+    m_counting = !(min_count == 1 && max_count == std::numeric_limits<size_t>::max ());
+    tl_assert (!m_counting || mode != EdgesOutside);
   }
 
   void finish (const db::Edge *o, size_t p)
   {
-    if (p == 0 && m_mode == EdgesOutside && m_seen.find (o) == m_seen.end ()) {
-      mp_output->insert (*o);
+    if (p != 0) {
+      return;
+    }
+
+    if (m_counting) {
+
+      size_t count = 0;
+      auto i = m_counts.find (o);
+      if (i != m_counts.end ()) {
+        count = i->second;
+      }
+
+      bool match = (count >= m_min_count && count <= m_max_count);
+      if (match == (m_mode != EdgesOutside)) {
+        mp_output->insert (*o);
+      }
+
+    } else {
+
+      if (m_mode == EdgesOutside && m_seen.find (o) == m_seen.end ()) {
+        mp_output->insert (*o);
+      }
+
     }
   }
 
@@ -363,14 +389,22 @@ public:
       if ((m_mode == EdgesInteract && db::edge_interacts (*o, *oo)) ||
           (m_mode == EdgesInside && db::edge_is_inside (*o, *oo))) {
 
-        if (m_seen.insert (o).second) {
-          mp_output->insert (*o);
+        if (m_counting) {
+          m_counts[o] += 1;
+        } else {
+          if (m_seen.insert (o).second) {
+            mp_output->insert (*o);
+          }
         }
 
       } else if (m_mode == EdgesOutside && ! db::edge_is_outside (*o, *oo)) {
 
         //  In this case we need to collect edges which are outside always - we report those on "finished".
-        m_seen.insert (o);
+        if (m_counting) {
+          m_counts[o] += 1;
+        } else {
+          m_seen.insert (o);
+        }
 
       }
 
@@ -380,7 +414,10 @@ public:
 private:
   OutputContainer *mp_output;
   std::set<const db::Edge *> m_seen;
+  std::map<const db::Edge *, size_t> m_counts;
   EdgeInteractionMode m_mode;
+  size_t m_min_count, m_max_count;
+  bool m_counting;
 };
 
 /**
@@ -408,20 +445,39 @@ DB_PUBLIC bool edge_is_outside (const db::Edge &a, const db::Polygon &b);
  *  There is a special box converter which is able to sort that out as well.
  */
 template <class OutputContainer, class OutputType = typename OutputContainer::value_type>
-class edge_to_region_interaction_filter
+class edge_to_polygon_interaction_filter
   : public db::box_scanner_receiver2<db::Edge, size_t, db::Polygon, size_t>
 {
 public:
-  edge_to_region_interaction_filter (OutputContainer *output, EdgeInteractionMode mode)
-    : mp_output (output), m_mode (mode)
+  edge_to_polygon_interaction_filter (OutputContainer *output, EdgeInteractionMode mode, size_t min_count, size_t max_count)
+    : mp_output (output), m_mode (mode), m_min_count (min_count), m_max_count (max_count)
   {
-    //  .. nothing yet ..
+    //  NOTE: "counting" does not really make much sense in Outside mode ...
+    m_counting = !(min_count == 1 && max_count == std::numeric_limits<size_t>::max ());
+    tl_assert (!m_counting || mode != EdgesOutside);
   }
 
   void finish (const OutputType *o)
   {
-    if (m_mode == EdgesOutside && m_seen.find (o) == m_seen.end ()) {
-      mp_output->insert (*o);
+    if (m_counting) {
+
+      size_t count = 0;
+      auto i = m_counts.find (o);
+      if (i != m_counts.end ()) {
+        count = i->second;
+      }
+
+      bool match = (count >= m_min_count && count <= m_max_count);
+      if (match == (m_mode != EdgesOutside)) {
+        mp_output->insert (*o);
+      }
+
+    } else {
+
+      if (m_mode == EdgesOutside && m_seen.find (o) == m_seen.end ()) {
+        mp_output->insert (*o);
+      }
+
     }
   }
 
@@ -448,7 +504,18 @@ public:
     const OutputType *ep = 0;
     tl::select (ep, e, p);
 
-    if (m_seen.find (ep) == m_seen.end ()) {
+    if (m_counting) {
+
+      if ((m_mode == EdgesInteract && db::edge_interacts (*e, *p)) ||
+          (m_mode == EdgesInside && db::edge_is_inside (*e, *p)) ||
+          (m_mode == EdgesOutside && ! db::edge_is_outside (*e, *p))) {
+
+        //  we report the result on "finish" here.
+        m_counts[ep] += 1;
+
+      }
+
+    } else if (m_seen.find (ep) == m_seen.end ()) {
 
       if ((m_mode == EdgesInteract && db::edge_interacts (*e, *p)) ||
           (m_mode == EdgesInside && db::edge_is_inside (*e, *p))) {
@@ -468,8 +535,11 @@ public:
 
 private:
   OutputContainer *mp_output;
+  std::map<const OutputType *, size_t> m_counts;
   std::set<const OutputType *> m_seen;
   EdgeInteractionMode m_mode;
+  size_t m_min_count, m_max_count;
+  bool m_counting;
 };
 
 /**
