@@ -395,12 +395,18 @@ void LayoutToNetlistStandardReader::read_netlist (db::Netlist *netlist, db::Layo
         throw tl::Exception (tl::to_string (tr ("Duplicate definition of device class: ")) + class_name);
       }
 
-      db::DeviceClassTemplateBase *dct = db::DeviceClassTemplateBase::template_by_name (templ_name);
-      if (! dct) {
-        throw tl::Exception (tl::to_string (tr ("Invalid device class template: ")) + templ_name);
+      db::DeviceClass *dc;
+      if (templ_name.empty ()) {
+        //  generic device class (issue #1696)
+        dc = new db::DeviceClass ();
+      } else {
+        db::DeviceClassTemplateBase *dct = db::DeviceClassTemplateBase::template_by_name (templ_name);
+        if (! dct) {
+          throw tl::Exception (tl::to_string (tr ("Invalid device class template: ")) + templ_name);
+        }
+        dc = dct->create ();
       }
 
-      db::DeviceClass *dc = dct->create ();
       dc->set_name (class_name);
       netlist->add_device_class (dc);
 
@@ -462,12 +468,17 @@ void LayoutToNetlistStandardReader::read_netlist (db::Netlist *netlist, db::Layo
       }
       br.done ();
 
-    } else if (l2n && (test (skeys::message_key) || test (lkeys::message_key))) {
+    } else if (l2n && (test (skeys::softconnect_key) || test (lkeys::softconnect_key))) {
 
-      db::LogEntryData data;
-      read_message_entry (data);
-
-      l2n->log_entry (data);
+      Brace br (this);
+      std::string l1;
+      read_word_or_quoted (l1);
+      while (br) {
+        std::string l2;
+        read_word_or_quoted (l2);
+        l2n->soft_connect (layer_by_name (l2n, l1), layer_by_name (l2n, l2));
+      }
+      br.done ();
 
     } else if (l2n && (test (skeys::global_key) || test (lkeys::global_key))) {
 
@@ -481,6 +492,25 @@ void LayoutToNetlistStandardReader::read_netlist (db::Netlist *netlist, db::Layo
       }
       br.done ();
 
+    } else if (l2n && (test (skeys::softglobal_key) || test (lkeys::softglobal_key))) {
+
+      Brace br (this);
+      std::string l1;
+      read_word_or_quoted (l1);
+      while (br) {
+        std::string g;
+        read_word_or_quoted (g);
+        l2n->soft_connect_global (layer_by_name (l2n, l1), g);
+      }
+      br.done ();
+
+    } else if (l2n && (test (skeys::message_key) || test (lkeys::message_key))) {
+
+      db::LogEntryData data;
+      read_message_entry (data);
+
+      l2n->log_entry (data);
+
     } else if (test (skeys::circuit_key) || test (lkeys::circuit_key)) {
 
       Brace br (this);
