@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -67,7 +67,7 @@ namespace db
  *                                       Content is the Netlist dump (reduced version of LayoutToNetlist)
  *
  *  [xrefs]:
- *    xref([xref|any]*)                - cross-reference part [short key: Z]
+ *    xref([xref|log|any]*)            - cross-reference part [short key: Z]
  *
  *  [xref]:
  *    circuit([non] [non] [status|message|log|circuit-xrefs|any]*)
@@ -82,11 +82,6 @@ namespace db
  *  [log-entry]:
  *    entry([severity] [message|any]*) - log entry [short key: M]
  *
- *  [severity]:
- *    info |                           - [short key: I]
- *    warning |                        - [short key: W]
- *    error                            - [short key: E]
- *
  *  [circuit-xrefs]:
  *    xref([xref-pin|xref-device|xref-circuit|xref-net|any]*)
  *                                     - circuit cross-reference part [short key: Z]
@@ -113,6 +108,11 @@ namespace db
  *  [message]:
  *    description(<name>)              - error description [short key: B]
  *
+ *  [severity]:
+ *    info |                           - [short key: I]
+ *    warning |                        - [short key: W]
+ *    error                            - [short key: E]
+ *
  *  [status]:
  *    mismatch |                       - [short key: 0]
  *    match |                          - [short key: 1]
@@ -145,10 +145,6 @@ namespace lvs_std_format
     static std::string nomatch_key;
     static std::string warning_key;
     static std::string skipped_key;
-
-    static std::string info_severity_key;
-    static std::string warning_severity_key;
-    static std::string error_severity_key;
   };
 
   struct DB_PUBLIC LongKeys
@@ -166,10 +162,6 @@ namespace lvs_std_format
     static std::string nomatch_key;
     static std::string warning_key;
     static std::string skipped_key;
-
-    static std::string info_severity_key;
-    static std::string warning_severity_key;
-    static std::string error_severity_key;
   };
 
   template <bool Short> struct DB_PUBLIC keys;
