@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -31,31 +31,34 @@
 namespace db
 {
 
+//  Switch for version-agnostic code
+#define KLAYOUT_META_INFO_V2
+
 /**
  *  @brief A structure describing the meta information from the reader
  *
  *  In the meta information block, the reader provides additional information
  *  about the file and content etc.
- *  "name" is a unique name that can be used to identify the information.
  *  "description" is a "speaking" description of the information.
  *  "value" is the value of the specific part of meta information.
  */
 struct DB_PUBLIC MetaInfo
 {
-  MetaInfo (const std::string &n, const std::string &d, const std::string &v)
-    : name (n), description (d), value (v)
+  MetaInfo (const std::string &d, const tl::Variant &v, bool p = false)
+    : description (d), value (v), persisted (p)
   {
     //  .. nothing else ..
   }
 
   MetaInfo ()
+    : persisted (false)
   {
     //  .. nothing else ..
   }
 
-  std::string name;
   std::string description;
-  std::string value;
+  tl::Variant value;
+  bool persisted;
 };
 
 /**
@@ -63,7 +66,6 @@ struct DB_PUBLIC MetaInfo
  */
 inline void mem_stat (MemStatistics *stat, MemStatistics::purpose_t purpose, int cat, const MetaInfo &v, bool no_self, void *parent)
 {
-  db::mem_stat (stat, purpose, cat, v.name, no_self, parent);
   db::mem_stat (stat, purpose, cat, v.description, no_self, parent);
   db::mem_stat (stat, purpose, cat, v.value, no_self, parent);
 }
