@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -235,6 +235,7 @@ public:
 
 private:
   friend class Net;
+  friend class Circuit;
 
   size_t m_pin_id;
   Net *mp_net;
@@ -246,6 +247,14 @@ private:
   {
     mp_net = net;
   }
+
+  /**
+   *  @brief Sets the pin ID
+   */
+  void set_pin_id (size_t id)
+  {
+    m_pin_id = id;
+  }
 };
 
 /**
@@ -343,6 +352,7 @@ public:
 
 private:
   friend class Net;
+  friend class SubCircuit;
 
   size_t m_pin_id;
   SubCircuit *mp_subcircuit;
@@ -355,6 +365,14 @@ private:
   {
     mp_net = net;
   }
+
+  /**
+   *  @brief Sets the pin ID
+   */
+  void set_pin_id (size_t pin)
+  {
+    m_pin_id = pin;
+  }
 };
 
 /**
