@@ -228,8 +228,8 @@ NetlistExtractor::extract_nets (const db::DeepShapeStore &dss, unsigned int layo
 
     const db::Cell &cell = mp_layout->cell (*cid);
 
-    const connected_clusters_type &clusters = mp_clusters->clusters_per_cell (*cid);
-    if (clusters.empty ()) {
+    connected_clusters_type &per_cell_clusters = mp_clusters->clusters_per_cell (*cid);
+    if (per_cell_clusters.empty ()) {
 
       bool any_good = false;
 
@@ -251,7 +251,7 @@ NetlistExtractor::extract_nets (const db::DeepShapeStore &dss, unsigned int layo
     if (dm) {
       //  This is a device abstract cell:
       //  make the terminal to cluster ID connections for the device abstract from the device cells
-      make_device_abstract_connections (dm, clusters);
+      make_device_abstract_connections (dm, per_cell_clusters);
       continue;
     }
 
@@ -284,11 +284,13 @@ NetlistExtractor::extract_nets (const db::DeepShapeStore &dss, unsigned int layo
 
     }
 
-    for (connected_clusters_type::all_iterator c = clusters.begin_all (); ! c.at_end (); ++c) {
+    for (connected_clusters_type::all_iterator c = per_cell_clusters.begin_all (); ! c.at_end (); ++c) {
 
-      const db::local_cluster<db::NetShape> &lc = clusters.cluster_by_id (*c);
-      const connected_clusters_type::connections_type &cc = clusters.connections_for_cluster (*c);
-      if (cc.empty () && lc.empty ()) {
+      const db::local_cluster<db::NetShape> &lc = per_cell_clusters.cluster_by_id (*c);
+      const connected_clusters_type::connections_type &cc = per_cell_clusters.connections_for_cluster (*c);
+      const std::set<size_t> &sc_up = per_cell_clusters.upward_soft_connections (*c);
+      const std::set<size_t> &sc_down = per_cell_clusters.downward_soft_connections (*c);
+      if (cc.empty () && sc_up.empty () && sc_down.empty () && lc.empty ()) {
         //  this is an entirely empty cluster so we skip it.
         //  Such clusters are left over when joining clusters.
         continue;
@@ -299,14 +301,14 @@ NetlistExtractor::extract_nets (const db::DeepShapeStore &dss, unsigned int layo
       circuit->add_net (net);
 
       //  make subcircuit connections (also make the subcircuits if required) from the connections of the clusters
-      make_and_connect_subcircuits (circuit, clusters, *c, net, subcircuits, circuits, pins_per_cluster_per_cell);
+      make_and_connect_subcircuits (circuit, per_cell_clusters, *c, net, subcircuits, circuits, pins_per_cluster_per_cell);
 
       //  connect devices
-      connect_devices (circuit, clusters, *c, net);
+      connect_devices (circuit, per_cell_clusters, *c, net);
 
       //  collect labels to net names
       std::set<std::string> net_names;
-      collect_labels (clusters, *c, net_names);
+      collect_labels (per_cell_clusters, *c, net_names);
 
       //  add the global names as second priority
       if (net_names.empty ()) {
@@ -321,7 +323,7 @@ NetlistExtractor::extract_nets (const db::DeepShapeStore &dss, unsigned int layo
       //  made to satisfy the subcircuit's pin, but not to make a physical connection.
       //  Don't know whether this is a good idea, so this code is disabled for now.
 
-      if (net_names.empty () && clusters.is_dummy (*c) && net->subcircuit_pin_count () == 1) {
+      if (net_names.empty () && per_cell_clusters.is_dummy (*c) && net->subcircuit_pin_count () == 1) {
         //  in the case of a dummy connection (partially connected subcircuits) create a
         //  new name indicating the subcircuit and the subcircuit net name - this makes subcircuit
         //  net names available (the net is pseudo-root inside in the subcircuit)
@@ -335,7 +337,7 @@ NetlistExtractor::extract_nets (const db::DeepShapeStore &dss, unsigned int layo
 
       assign_net_names (net, net_names);
 
-      if (! clusters.is_root (*c)) {
+      if (! per_cell_clusters.is_root (*c)) {
         //  a non-root cluster makes a pin
         size_t pin_id = make_pin (circuit, net);
         c2p.insert (std::make_pair (*c, pin_id));
@@ -362,28 +364,73 @@ NetlistExtractor::assign_net_names (db::Net *net, const std::set<std::string> &n
   net->set_name (nn);
 }
 
+static void
+collect_soft_connected_clusters (size_t from_id, const NetlistExtractor::connected_clusters_type &clusters, std::set<size_t> &ids)
+{
+  if (ids.find (from_id) != ids.end ()) {
+    return;
+  }
+
+  ids.insert (from_id);
+
+  auto upward = clusters.upward_soft_connections (from_id);
+  for (auto i = upward.begin (); i != upward.end (); ++i) {
+    collect_soft_connected_clusters (*i, clusters, ids);
+  }
+
+  auto downward = clusters.downward_soft_connections (from_id);
+  for (auto i = downward.begin (); i != downward.end (); ++i) {
+    collect_soft_connected_clusters (*i, clusters, ids);
+  }
+}
+
 void
-NetlistExtractor::make_device_abstract_connections (db::DeviceAbstract *dm, const connected_clusters_type &clusters)
+NetlistExtractor::make_device_abstract_connections (db::DeviceAbstract *dm, connected_clusters_type &clusters)
 {
   //  make the terminal to cluster ID connections for the device abstract from the device cells
 
   if (m_terminal_annot_name_id.first) {
 
-    for (connected_clusters_type::const_iterator dc = clusters.begin (); dc != clusters.end (); ++dc) {
+    for (connected_clusters_type::iterator dc = clusters.begin (); dc != clusters.end (); ++dc) {
 
-      for (local_cluster_type::attr_iterator a = dc->begin_attr (); a != dc->end_attr (); ++a) {
+      std::set<size_t> ids;
+      collect_soft_connected_clusters (dc->id (), clusters, ids);
+
+      for (auto id = ids.begin (); id != ids.end (); ++id) {
+
+        const local_cluster_type &lc = clusters.cluster_by_id (*id);
+        bool join = false;
+
+        for (local_cluster_type::attr_iterator a = lc.begin_attr (); a != lc.end_attr (); ++a) {
+
+          if (! db::is_prop_id_attr (*a)) {
+            continue;
+          }
+
+          db::properties_id_type pi = db::prop_id_from_attr (*a);
+
+          const db::PropertiesRepository::properties_set &ps = mp_layout->properties_repository ().properties (pi);
+          for (db::PropertiesRepository::properties_set::const_iterator j = ps.begin (); j != ps.end (); ++j) {
+
+            if (j->first == m_terminal_annot_name_id.second) {
+
+              size_t terminal_id = j->second.to<size_t> ();
+              if (*id != dc->id ()) {
+                tl::warn << tl::sprintf (tl::to_string (tr ("Ignoring soft connection at device terminal %s for device %s")), dm->device_class ()->terminal_definition (terminal_id)->name (), dm->device_class ()->name ());
+                join = true;
+              }
+
+              dm->set_cluster_id_for_terminal (terminal_id, dc->id ());
+
+            }
+
+          }
 
-        if (! db::is_prop_id_attr (*a)) {
-          continue;
         }
 
-        db::properties_id_type pi = db::prop_id_from_attr (*a);
-
-        const db::PropertiesRepository::properties_set &ps = mp_layout->properties_repository ().properties (pi);
-        for (db::PropertiesRepository::properties_set::const_iterator j = ps.begin (); j != ps.end (); ++j) {
-          if (j->first == m_terminal_annot_name_id.second) {
-            dm->set_cluster_id_for_terminal (j->second.to<size_t> (), dc->id ());
-          }
+        if (join) {
+          //  copy the terminal attributes and shapes so we attach the terminal here in the device connection step
+          clusters.join_cluster_with (dc->id (), *id);
         }
 
       }
