@@ -633,6 +633,13 @@ SpiceCircuitDict::get_line ()
 
       } else if (ex.test_without_case (".endl")) {
 
+        std::string name;
+        if (ex.try_read_name (name)) {
+          if (! m_in_lib.empty () && mp_netlist->normalize_name (name) != m_in_lib.back ()) {
+            warn (tl::sprintf (tl::to_string (tr (".endl has wrong name: %s given, %s expected")), mp_netlist->normalize_name (name), m_in_lib.back ()));
+          }
+        }
+
         if (! m_in_lib.empty ()) {
           m_in_lib.pop_back ();
         } else {
@@ -936,6 +943,12 @@ SpiceNetlistBuilder::circuit_for (const SpiceCachedCircuit *cc, const parameters
   if (cp == c->second.end ()) {
     return 0;
   }
+
+  //  a null pointer indicates that we are currently defining this circuit
+  if (cp->second == 0) {
+    error (tl::sprintf (tl::to_string (tr ("Subcircuit '%s' called recursively")), cc->name ()));
+  }
+
   return cp->second;
 }
 
@@ -1048,7 +1061,8 @@ SpiceNetlistBuilder::build_circuit (const SpiceCachedCircuit *cc, const paramete
     c->set_name (make_circuit_name (cc->name (), pv));
   }
 
-  register_circuit_for (cc, pv, c, anonymous_top_level);
+  //  pre-register the circuit - allows detecting recursive calls
+  register_circuit_for (cc, pv, 0, false);
 
   std::unique_ptr<std::map<std::string, db::Net *> > n2n (mp_nets_by_name.release ());
   mp_nets_by_name.reset (0);
@@ -1088,6 +1102,14 @@ SpiceNetlistBuilder::build_circuit (const SpiceCachedCircuit *cc, const paramete
   std::swap (c, mp_netlist_circuit);
   std::swap (vars, m_variables);
 
+  //  final registration if required
+  if (! anonymous_top_level || ! c->is_empty ()) {
+    register_circuit_for (cc, pv, c, anonymous_top_level);
+  } else {
+    mp_netlist->remove_circuit (c);
+    c = 0;
+  }
+
   return c;
 }
 
@@ -1196,7 +1218,10 @@ SpiceNetlistBuilder::process_element (tl::Extractor &ex, const std::string &pref
         cc_nc->set_anonymous (true);
         cc = cc_nc;
         std::vector<std::string> pins;
-        pins.resize (nn.size ());
+        pins.reserve (nn.size ());
+        for (size_t i = 0; i < nn.size (); ++i) {
+          pins.push_back (tl::to_string (i + 1));
+        }
         cc_nc->set_pins (pins);
       }
     }
@@ -1211,7 +1236,7 @@ SpiceNetlistBuilder::process_element (tl::Extractor &ex, const std::string &pref
     }
 
     if (cc->pin_count () != nn.size ()) {
-      error (tl::sprintf (tl::to_string (tr ("Pin count mismatch between circuit definition and circuit call: %d expected, got %d")), int (cc->pin_count ()), int (nets.size ())));
+      error (tl::sprintf (tl::to_string (tr ("Pin count mismatch between circuit definition and circuit call: %d expected, got %d (definition maybe implicit somewhere before)")), int (cc->pin_count ()), int (nets.size ())));
     }
 
     db::Circuit *c = build_circuit (cc, pv);
