@@ -199,7 +199,7 @@ public:
    *  Creates a region from a recursive shape iterator. This allows feeding a region
    *  from a hierarchy of cells.
    */
-  explicit Region (const RecursiveShapeIterator &si);
+  explicit Region (const RecursiveShapeIterator &si, bool merged_semantics = true, bool is_merged = false);
 
   /**
    *  @brief Constructor from a RecursiveShapeIterator with a transformation
@@ -208,7 +208,23 @@ public:
    *  from a hierarchy of cells. The transformation is useful to scale to a specific
    *  DBU for example.
    */
-  explicit Region (const RecursiveShapeIterator &si, const db::ICplxTrans &trans, bool merged_semantics = true);
+  explicit Region (const RecursiveShapeIterator &si, const db::ICplxTrans &trans, bool merged_semantics = true, bool is_merged = false);
+
+  /**
+   *  @brief Constructor from a Shapes container
+   *
+   *  Creates a region from a shapes container.
+   */
+  explicit Region (const Shapes &si, bool merged_semantics = true, bool is_merged = false);
+
+  /**
+   *  @brief Constructor from a Shapes container with a transformation
+   *
+   *  Creates a region from a recursive shape iterator. This allows feeding a region
+   *  from a hierarchy of cells. The transformation is useful to scale to a specific
+   *  DBU for example.
+   */
+  explicit Region (const Shapes &si, const db::ICplxTrans &trans, bool merged_semantics = true, bool is_merged = false);
 
   /**
    *  @brief Constructor from a RecursiveShapeIterator providing a deep representation
@@ -232,6 +248,14 @@ public:
    */
   explicit Region (DeepShapeStore &dss);
 
+  /**
+   *  @brief Writes the region to a file
+   *
+   *  This method is provided for debugging purposes. A flat image of the
+   *  region is written to a layout file with a single top cell on layer 0/0.
+   */
+  void write (const std::string &fn) const;
+
   /**
    *  @brief Implementation of the ShapeCollection interface
    */
@@ -770,7 +794,7 @@ public:
    */
   Edges edges () const
   {
-    return Edges (mp_delegate->edges (0));
+    return Edges (mp_delegate->edges (0, 0));
   }
 
   /**
@@ -783,7 +807,34 @@ public:
    */
   Edges edges (const EdgeFilterBase &filter) const
   {
-    return mp_delegate->edges (&filter);
+    return mp_delegate->edges (&filter, 0);
+  }
+
+  /**
+   *  @brief Returns an edge set containing all edges of the polygons in this region
+   *
+   *  Merged semantics applies. In merged semantics, only full, outer edges are delivered.
+   *  This version allows specifying a polygon to edge processor with additional features
+   *  like extraction of convex edges only.
+   */
+  Edges edges (const db::PolygonToEdgeProcessorBase &proc) const
+  {
+    return Edges (mp_delegate->edges (0, &proc));
+  }
+
+  /**
+   *  @brief Returns an edge set containing all edges of the polygons in this region
+   *
+   *  This version allows one to specify a filter by which the edges are filtered before they are
+   *  returned.
+   *
+   *  Merged semantics applies. In merged semantics, only full, outer edges are delivered.
+   *  This version allows specifying a polygon to edge processor with additional features
+   *  like extraction of convex edges only.
+   */
+  Edges edges (const EdgeFilterBase &filter, const db::PolygonToEdgeProcessorBase &proc) const
+  {
+    return mp_delegate->edges (&filter, &proc);
   }
 
   /**
