@@ -2651,6 +2651,16 @@ public:
    */
   box_type bbox () const;
 
+  /**
+   *  @brief Returns the box if the object represents a rectangle or an empty box if not
+   *
+   *  This method returns the rectangle (aka box) the shape represents a polygon
+   *  that is a rectangle, a path with two points and no rounded ends or an actual box.
+   *
+   *  If not, an empty box is returned.
+   */
+  box_type rectangle () const;
+
   /** 
    *  @brief Compute the area of the shape
    */
@@ -2759,6 +2769,11 @@ public:
     return m_trans < d.m_trans;
   }
 
+  /**
+   *  @brief Hash value
+   */
+  size_t hash_value () const;
+
   /** 
    *  @brief Convert to a string
    */
@@ -2827,6 +2842,21 @@ public:
 };
 
 }  // namespace db
+
+namespace std
+{
+
+  //  provide a template specialization for std::hash<T>
+  template <>
+  struct hash <db::Shape>
+  {
+    size_t operator() (const db::Shape &s) const
+    {
+      return s.hash_value ();
+    }
+  };
+
+}  // namespace std
   
 #endif
 
