@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -90,6 +90,24 @@ void SubCircuit::set_trans (const db::DCplxTrans &t)
   m_trans = t;
 }
 
+void SubCircuit::erase_pin (size_t pin_id)
+{
+  Net *net = net_for_pin (pin_id);
+
+  if (! tl::is_null_iterator (m_pin_refs [pin_id])) {
+    net->erase_subcircuit_pin (m_pin_refs [pin_id]);
+  }
+
+  m_pin_refs.erase (m_pin_refs.begin () + pin_id);
+
+  //  correct pin IDs for the pins with ID > pin_id
+  for (auto p = m_pin_refs.begin () + pin_id; p != m_pin_refs.end (); ++p) {
+    if (! tl::is_null_iterator (*p)) {
+      (*p)->set_pin_id ((*p)->pin_id () - 1);
+    }
+  }
+}
+
 void SubCircuit::set_pin_ref_for_pin (size_t pin_id, Net::subcircuit_pin_iterator iter)
 {
   if (m_pin_refs.size () < pin_id + 1) {
