@@ -467,6 +467,39 @@ public:
     }
   }
 
+  /**
+   *  @brief Gets the default grids
+   */
+  const std::string &default_grids () const
+  {
+    return m_default_grids;
+  }
+
+  /**
+   *  @brief Gets the default grids, parsed as a list of double values
+   */
+  std::vector<double> default_grid_list () const;
+
+  /**
+   *  @brief Gets the default grid (strong grid), parsed from the list
+   *
+   *  The default grid is the one marked with an exclamation mark in the
+   *  grid list (e.g. "0.01!,0.02,0.05"). If there is not such default
+   *  grid, this method returns zero.
+   */
+  double default_grid () const;
+
+  /**
+   *  @brief Sets the default default grids
+   */
+  void set_default_grids (const std::string &default_grids)
+  {
+    if (default_grids != m_default_grids) {
+      m_default_grids = default_grids;
+      technology_changed ();
+    }
+  }
+
   /**
    *  @brief Gets the layer properties file path (empty if none is specified)
    */
@@ -650,6 +683,7 @@ private:
   std::string m_name, m_description, m_group;
   std::string m_grain_name;
   double m_dbu;
+  std::string m_default_grids;
   std::string m_explicit_base_path, m_default_base_path;
   db::LoadLayoutOptions m_load_layout_options;
   db::SaveLayoutOptions m_save_layout_options;
