@@ -30,7 +30,9 @@
 #include "dbOriginalLayerTexts.h"
 #include "dbEdges.h"
 #include "dbRegion.h"
-
+#include "dbLayout.h"
+#include "dbWriter.h"
+#include "tlStream.h"
 #include "tlVariant.h"
 
 #include <sstream>
@@ -90,6 +92,23 @@ Texts::Texts (const RecursiveShapeIterator &si, DeepShapeStore &dss, const db::I
   mp_delegate = new DeepTexts (si, dss, trans);
 }
 
+void
+Texts::write (const std::string &fn) const
+{
+  //  method provided for debugging purposes
+
+  db::Layout layout;
+  const db::Cell &top = layout.cell (layout.add_cell ("TEXTS"));
+  unsigned int li = layout.insert_layer (db::LayerProperties (0, 0));
+  insert_into (&layout, top.cell_index (), li);
+
+  tl::OutputStream os (fn);
+  db::SaveLayoutOptions opt;
+  opt.set_format_from_filename (fn);
+  db::Writer writer (opt);
+  writer.write (layout, os);
+}
+
 template <class Sh>
 void Texts::insert (const Sh &shape)
 {
@@ -199,6 +218,11 @@ MutableTexts *Texts::mutable_texts ()
   return texts;
 }
 
+Texts Texts::processed (const TextProcessorBase &proc) const
+{
+  return Texts (mp_delegate->processed (proc));
+}
+
 void Texts::processed (Region &output, const TextToPolygonProcessorBase &filter) const
 {
   output = Region (mp_delegate->processed_to_polygons (filter));
