@@ -289,13 +289,13 @@ Class<db::LayoutToNetlist> decl_dbLayoutToNetlist ("db", "LayoutToNetlist",
   gsi::method ("name", (const std::string &(db::LayoutToNetlist::*) () const) &db::LayoutToNetlist::name,
     "@brief Gets the name of the database\n"
   ) +
-  gsi::method ("name=", &db::LayoutToNetlist::set_name,
+  gsi::method ("name=", &db::LayoutToNetlist::set_name, gsi::arg ("name"),
     "@brief Sets the name of the database\n"
   ) +
   gsi::method ("description", (const std::string &(db::LayoutToNetlist::*) () const) &db::LayoutToNetlist::name,
     "@brief Gets the description of the database\n"
   ) +
-  gsi::method ("description=", &db::LayoutToNetlist::set_name,
+  gsi::method ("description=", &db::LayoutToNetlist::set_name, gsi::arg ("description"),
     "@brief Sets the description of the database\n"
   ) +
   gsi::method ("filename", &db::LayoutToNetlist::filename,
@@ -306,7 +306,7 @@ Class<db::LayoutToNetlist> decl_dbLayoutToNetlist ("db", "LayoutToNetlist",
     "@brief Gets the original file name of the database\n"
     "The original filename is the layout file from which the netlist DB was created."
   ) +
-  gsi::method ("original_file=", &db::LayoutToNetlist::set_original_file,
+  gsi::method ("original_file=", &db::LayoutToNetlist::set_original_file, gsi::arg ("path"),
     "@brief Sets the original file name of the database\n"
   ) +
   gsi::method ("layer_name", (std::string (db::LayoutToNetlist::*) (const db::ShapeCollection &region) const) &db::LayoutToNetlist::name, gsi::arg ("l"),
@@ -433,6 +433,32 @@ Class<db::LayoutToNetlist> decl_dbLayoutToNetlist ("db", "LayoutToNetlist",
     "\n"
     "This variant has been introduced in version 0.27.\n"
   ) +
+  gsi::method ("soft_connect", (void (db::LayoutToNetlist::*) (const db::Region &, const db::Region &)) &db::LayoutToNetlist::soft_connect, gsi::arg ("a"), gsi::arg ("b"),
+    "@brief Defines an inter-layer connection for the given layers in soft mode.\n"
+    "Connects two layers through a soft connection.\n"
+    "Soft connections cannot make connections between two different nets.\n"
+    "These are directional connections where 'b' is the 'lower' layer (typically high-ohmic substrate or diffusion).\n"
+    "\n"
+    "Soft connections have been introduced in version 0.29.\n"
+  ) +
+  gsi::method ("soft_connect", (void (db::LayoutToNetlist::*) (const db::Region &, const db::Texts &)) &db::LayoutToNetlist::soft_connect, gsi::arg ("a"), gsi::arg ("b"),
+    "@brief Defines an inter-layer connection for the given layers in soft mode.\n"
+    "Connects two layers through a soft connection.\n"
+    "Soft connections cannot make connections between two different nets.\n"
+    "These are directional connections where 'b' is the 'lower' layer (typically high-ohmic substrate or diffusion).\n"
+    "As one argument is a (hierarchical) text collection, this method is used to attach net labels to polygons.\n"
+    "\n"
+    "Soft connections have been introduced in version 0.29.\n"
+  ) +
+  gsi::method ("soft_connect", (void (db::LayoutToNetlist::*) (const db::Texts &, const db::Region &)) &db::LayoutToNetlist::soft_connect, gsi::arg ("a"), gsi::arg ("b"),
+    "@brief Defines an inter-layer connection for the given layers in soft mode.\n"
+    "Connects two layers through a soft connection.\n"
+    "Soft connections cannot make connections between two different nets.\n"
+    "These are directional connections where 'b' is the 'lower' layer (typically high-ohmic substrate or diffusion).\n"
+    "As one argument is a (hierarchical) text collection, this method is used to attach net labels to polygons.\n"
+    "\n"
+    "Soft connections have been introduced in version 0.29.\n"
+  ) +
   gsi::method ("connect_global", (size_t (db::LayoutToNetlist::*) (const db::Region &, const std::string &)) &db::LayoutToNetlist::connect_global, gsi::arg ("l"), gsi::arg ("global_net_name"),
     "@brief Defines a connection of the given layer with a global net.\n"
     "This method returns the ID of the global net. Use \\global_net_name to get "
@@ -441,10 +467,26 @@ Class<db::LayoutToNetlist> decl_dbLayoutToNetlist ("db", "LayoutToNetlist",
   gsi::method ("connect_global", (size_t (db::LayoutToNetlist::*) (const db::Texts &, const std::string &)) &db::LayoutToNetlist::connect_global, gsi::arg ("l"), gsi::arg ("global_net_name"),
     "@brief Defines a connection of the given text layer with a global net.\n"
     "This method returns the ID of the global net. Use \\global_net_name to get "
-    "the name back from the ID."
+    "the name back from the ID.\n"
     "\n"
     "This variant has been introduced in version 0.27.\n"
   ) +
+  gsi::method ("soft_connect_global", (size_t (db::LayoutToNetlist::*) (const db::Region &, const std::string &)) &db::LayoutToNetlist::soft_connect_global, gsi::arg ("l"), gsi::arg ("global_net_name"),
+    "@brief Defines a connection of the given layer with a global net in soft mode.\n"
+    "This method returns the ID of the global net. Use \\global_net_name to get "
+    "the name back from the ID.\n"
+    "Soft connections are directional, where the global net is the 'lower' layer (typically high-ohmic substrate or diffusion).\n"
+    "\n"
+    "Soft connections have been introduced in version 0.29.\n"
+  ) +
+  gsi::method ("soft_connect_global", (size_t (db::LayoutToNetlist::*) (const db::Texts &, const std::string &)) &db::LayoutToNetlist::soft_connect_global, gsi::arg ("l"), gsi::arg ("global_net_name"),
+    "@brief Defines a connection of the given text layer with a global net in soft mode.\n"
+    "This method returns the ID of the global net. Use \\global_net_name to get "
+    "the name back from the ID.\n"
+    "Soft connections are directional, where the global net is the 'lower' layer (typically high-ohmic substrate or diffusion).\n"
+    "\n"
+    "Soft connections have been introduced in version 0.29.\n"
+  ) +
   gsi::method ("global_net_name", &db::LayoutToNetlist::global_net_name, gsi::arg ("global_net_id"),
     "@brief Gets the global net name for the given global net ID."
   ) +
@@ -464,6 +506,12 @@ Class<db::LayoutToNetlist> decl_dbLayoutToNetlist ("db", "LayoutToNetlist",
     "\n"
     "This attribute has been introduced in version 0.27.\n"
   ) +
+  gsi::method ("make_soft_connection_diodes=", &db::LayoutToNetlist::set_make_soft_connection_diodes, gsi::arg ("flag"),
+    "@hide"
+  ) +
+  gsi::method ("make_soft_connection_diodes", &db::LayoutToNetlist::make_soft_connection_diodes,
+    "@hide"
+  ) +
   gsi::method ("top_level_mode=", &db::LayoutToNetlist::set_top_level_mode, gsi::arg ("flag"),
     "@brief Sets a flag indicating whether top level mode is enabled.\n"
     "\n"
