@@ -29,6 +29,7 @@
 #include "dbPCellDeclaration.h"
 #include "dbLibrary.h"
 #include "dbLibraryManager.h"
+#include "tlLog.h"
 
 namespace gsi
 {
@@ -217,12 +218,12 @@ LibraryClass decl_Library ("db", "Library",
   ) +
   gsi::method ("is_for_technology", &db::Library::is_for_technology, gsi::arg ("tech"),
     "@brief Returns a value indicating whether the library is associated with the given technology.\n"
+    "The method is equivalent to checking whether the \\technologies list is empty.\n"
+    "\n"
     "This method has been introduced in version 0.27"
   ) +
   gsi::method ("for_technologies", &db::Library::for_technologies,
     "@brief Returns a value indicating whether the library is associated with any technology.\n"
-    "The method is equivalent to checking whether the \\technologies list is empty.\n"
-    "\n"
     "This method has been introduced in version 0.27"
   ) +
   gsi::method ("technologies", &db::Library::get_technologies,
@@ -701,23 +702,23 @@ Class<PCellDeclarationImpl> decl_PCellDeclaration (decl_PCellDeclaration_Native,
 // ---------------------------------------------------------------
 //  db::PCellParameterDeclaration binding
 
-unsigned int get_type (const db::PCellParameterDeclaration *pd)
+static unsigned int get_type (const db::PCellParameterDeclaration *pd)
 {
   return (unsigned int) pd->get_type ();
 }
 
-void set_type (db::PCellParameterDeclaration *pd, unsigned int t)
+static void set_type (db::PCellParameterDeclaration *pd, unsigned int t)
 {
   pd->set_type (db::PCellParameterDeclaration::type (t));
 }
 
-void clear_choices (db::PCellParameterDeclaration *pd)
+static void clear_choices (db::PCellParameterDeclaration *pd)
 {
   pd->set_choices (std::vector<tl::Variant> ());
   pd->set_choice_descriptions (std::vector<std::string> ());
 }
 
-void add_choice (db::PCellParameterDeclaration *pd, const std::string &d, const tl::Variant &v)
+static void add_choice (db::PCellParameterDeclaration *pd, const std::string &d, const tl::Variant &v)
 {
   std::vector<tl::Variant> vv = pd->get_choices ();
   std::vector<std::string> dd = pd->get_choice_descriptions ();
@@ -772,26 +773,7 @@ static unsigned int pd_type_none ()
   return (unsigned int) db::PCellParameterDeclaration::t_none;
 }
 
-db::PCellParameterDeclaration *ctor_pcell_parameter (const std::string &name, unsigned int type, const std::string &description)
-{
-  db::PCellParameterDeclaration *pd = new db::PCellParameterDeclaration ();
-  pd->set_name (name);
-  pd->set_type (db::PCellParameterDeclaration::type (type));
-  pd->set_description (description);
-  return pd;
-}
-
-db::PCellParameterDeclaration *ctor_pcell_parameter_2 (const std::string &name, unsigned int type, const std::string &description, const tl::Variant &def)
-{
-  db::PCellParameterDeclaration *pd = new db::PCellParameterDeclaration ();
-  pd->set_name (name);
-  pd->set_type (db::PCellParameterDeclaration::type (type));
-  pd->set_description (description);
-  pd->set_default (def);
-  return pd;
-}
-
-db::PCellParameterDeclaration *ctor_pcell_parameter_3 (const std::string &name, unsigned int type, const std::string &description, const tl::Variant &def, const std::string &unit)
+db::PCellParameterDeclaration *ctor_pcell_parameter (const std::string &name, unsigned int type, const std::string &description, const tl::Variant &def, const std::string &unit)
 {
   db::PCellParameterDeclaration *pd = new db::PCellParameterDeclaration ();
   pd->set_name (name);
@@ -803,20 +785,7 @@ db::PCellParameterDeclaration *ctor_pcell_parameter_3 (const std::string &name,
 }
 
 Class<db::PCellParameterDeclaration> decl_PCellParameterDeclaration ("db", "PCellParameterDeclaration",
-  gsi::constructor ("new", &ctor_pcell_parameter, gsi::arg ("name"), gsi::arg ("type"), gsi::arg ("description"),
-    "@brief Create a new parameter declaration with the given name and type\n"
-    "@param name The parameter name\n"
-    "@param type One of the Type... constants describing the type of the parameter\n"
-    "@param description The description text\n"
-  ) +
-  gsi::constructor ("new", &ctor_pcell_parameter_2, gsi::arg ("name"), gsi::arg ("type"), gsi::arg ("description"), gsi::arg ("default"),
-    "@brief Create a new parameter declaration with the given name, type and default value\n"
-    "@param name The parameter name\n"
-    "@param type One of the Type... constants describing the type of the parameter\n"
-    "@param description The description text\n"
-    "@param default The default (initial) value\n"
-  ) +
-  gsi::constructor ("new", &ctor_pcell_parameter_3, gsi::arg ("name"), gsi::arg ("type"), gsi::arg ("description"), gsi::arg ("default"), gsi::arg ("unit"),
+  gsi::constructor ("new", &ctor_pcell_parameter, gsi::arg ("name"), gsi::arg ("type"), gsi::arg ("description"), gsi::arg ("default", tl::Variant (), "nil"), gsi::arg ("unit", std::string ()),
     "@brief Create a new parameter declaration with the given name, type, default value and unit string\n"
     "@param name The parameter name\n"
     "@param type One of the Type... constants describing the type of the parameter\n"
@@ -881,6 +850,44 @@ Class<db::PCellParameterDeclaration> decl_PCellParameterDeclaration ("db", "PCel
   gsi::method ("choice_descriptions", &db::PCellParameterDeclaration::get_choice_descriptions,
     "@brief Returns a list of choice descriptions\n"
   ) +
+  gsi::method ("min_value", &db::PCellParameterDeclaration::min_value,
+    "@brief Gets the minimum value allowed\n"
+    "See \\min_value= for a description of this attribute.\n"
+    "\n"
+    "This attribute has been added in version 0.29."
+  ) +
+  gsi::method ("min_value=", &db::PCellParameterDeclaration::set_min_value, gsi::arg ("value"),
+    "@brief Sets the minimum value allowed\n"
+    "The minimum value is a visual feature and limits the allowed values for numerical\n"
+    "entry boxes. This applies to parameters of type int or double. The minimum value\n"
+    "is not effective if choices are present.\n"
+    "\n"
+    "The minimum value is not enforced - for example there is no restriction implemented\n"
+    "when setting values programmatically.\n"
+    "\n"
+    "Setting this attribute to \"nil\" (the default) implies \"no limit\".\n"
+    "\n"
+    "This attribute has been added in version 0.29."
+  ) +
+  gsi::method ("max_value", &db::PCellParameterDeclaration::max_value,
+    "@brief Gets the maximum value allowed\n"
+    "See \\max_value= for a description of this attribute.\n"
+    "\n"
+    "This attribute has been added in version 0.29."
+  ) +
+  gsi::method ("max_value=", &db::PCellParameterDeclaration::set_max_value, gsi::arg ("value"),
+    "@brief Sets the maximum value allowed\n"
+    "The maximum value is a visual feature and limits the allowed values for numerical\n"
+    "entry boxes. This applies to parameters of type int or double. The maximum value\n"
+    "is not effective if choices are present.\n"
+    "\n"
+    "The maximum value is not enforced - for example there is no restriction implemented\n"
+    "when setting values programmatically.\n"
+    "\n"
+    "Setting this attribute to \"nil\" (the default) implies \"no limit\".\n"
+    "\n"
+    "This attribute has been added in version 0.29."
+  ) +
   gsi::method ("default", &db::PCellParameterDeclaration::get_default,
     "@brief Gets the default value\n"
   ) +
