@@ -669,6 +669,26 @@ static tl::Variant get_dbox (const db::Shape *s)
   }
 }
 
+static tl::Variant get_rectangle (const db::Shape *s)
+{
+  db::Shape::box_type b = s->rectangle ();
+  if (! b.empty ()) {
+    return tl::Variant (b);
+  } else {
+    return tl::Variant ();
+  }
+}
+
+static tl::Variant get_drectangle (const db::Shape *s)
+{
+  db::Shape::box_type b = s->rectangle ();
+  if (! b.empty ()) {
+    return tl::Variant (db::CplxTrans (shape_dbu (s)) * b);
+  } else {
+    return tl::Variant ();
+  }
+}
+
 static tl::Variant get_edge (const db::Shape *s)
 {
   db::Shape::edge_type p;
@@ -1151,7 +1171,7 @@ Class<db::Shape> decl_Shape ("db", "Shape",
     "\n"
     "The \\Layout object can be used to retrieve the actual properties associated with the ID."
   ) +
-  gsi::method_ext ("prop_id=", &set_prop_id,
+  gsi::method_ext ("prop_id=", &set_prop_id, gsi::arg ("id"),
     "@brief Sets the properties ID of this shape\n"
     "\n"
     "The \\Layout object can be used to retrieve an ID for a given set of properties. "
@@ -1982,6 +2002,22 @@ Class<db::Shape> decl_Shape ("db", "Shape",
     "\n"
     "This method has been added in version 0.25.\n"
   ) +
+  gsi::method_ext ("rectangle", &get_rectangle,
+    "@brief Gets the rectangle if the object represents one or nil if not\n"
+    "\n"
+    "If the shape represents a rectangle - i.e. a box or box polygon, a path with two points and no round ends - "
+    "this method returns the box. If not, nil is returned.\n"
+    "\n"
+    "This method has been introduced in version 0.29."
+  ) +
+  gsi::method_ext ("drectangle", &get_drectangle,
+    "@brief Gets the rectangle in micron units if the object represents one or nil if not\n"
+    "\n"
+    "If the shape represents a rectangle - i.e. a box or box polygon, a path with two points and no round ends - "
+    "this method returns the box. If not, nil is returned.\n"
+    "\n"
+    "This method has been introduced in version 0.29."
+  ) +
   gsi::method ("is_user_object?", &db::Shape::is_user_object,
     "@brief Returns true if the shape is a user defined object\n"
   ) +
@@ -2095,6 +2131,21 @@ Class<db::Shape> decl_Shape ("db", "Shape",
     "Equality of shapes is not specified by the identity of the objects but by the\n"
     "identity of the pointers - both shapes must refer to the same object.\n"
   ) +
+  gsi::method ("<", &db::Shape::operator<, gsi::arg ("other"),
+    "@brief Less operator\n"
+    "\n"
+    "The less operator implementation is based on pointers and not strictly reproducible."
+    "However, it is good enough so Shape objects can serve as keys in hashes (see also \\hash).\n"
+    "\n"
+    "This method has been introduced in version 0.29.1."
+  ) +
+  gsi::method ("hash", &db::Shape::hash_value,
+    "@brief Hash function\n"
+    "\n"
+    "The hash function enables Shape objects as keys in hashes.\n"
+    "\n"
+    "This method has been introduced in version 0.29.1."
+  ) +
   gsi::method ("to_s", &db::Shape::to_string,
     "@brief Create a string showing the contents of the reference\n"
     "\n"
