@@ -33,6 +33,160 @@
 namespace gsi
 {
 
+// ---------------------------------------------------------------------------------
+//  TextFilter binding
+
+class TextFilterImpl
+  : public shape_filter_impl<db::TextFilterBase>
+{
+public:
+  TextFilterImpl () { }
+
+  bool issue_selected (const db::Text &) const
+  {
+    return false;
+  }
+
+  virtual bool selected (const db::Text &text) const
+  {
+    if (f_selected.can_issue ()) {
+      return f_selected.issue<TextFilterImpl, bool, const db::Text &> (&TextFilterImpl::issue_selected, text);
+    } else {
+      return issue_selected (text);
+    }
+  }
+
+  gsi::Callback f_selected;
+
+private:
+  //  No copying
+  TextFilterImpl &operator= (const TextFilterImpl &);
+  TextFilterImpl (const TextFilterImpl &);
+};
+
+Class<gsi::TextFilterImpl> decl_TextFilterImpl ("db", "TextFilter",
+  TextFilterImpl::method_decls (false) +
+  callback ("selected", &TextFilterImpl::issue_selected, &TextFilterImpl::f_selected, gsi::arg ("text"),
+    "@brief Selects a text\n"
+    "This method is the actual payload. It needs to be reimplemented in a derived class.\n"
+    "It needs to analyze the text and return 'true' if it should be kept and 'false' if it should be discarded."
+  ),
+  "@brief A generic text filter adaptor\n"
+  "\n"
+  "Text filters are an efficient way to filter texts from a Texts collection. To apply a filter, derive your own "
+  "filter class and pass an instance to \\Texts#filter or \\Texts#filtered method.\n"
+  "\n"
+  "Conceptually, these methods take each text from the collection and present it to the filter's 'selected' method.\n"
+  "Based on the result of this evaluation, the text is kept or discarded.\n"
+  "\n"
+  "The magic happens when deep mode text collections are involved. In that case, the filter will use as few calls as possible "
+  "and exploit the hierarchical compression if possible. It needs to know however, how the filter behaves. You "
+  "need to configure the filter by calling \\is_isotropic, \\is_scale_invariant or \\is_isotropic_and_scale_invariant "
+  "before using the filter.\n"
+  "\n"
+  "You can skip this step, but the filter algorithm will assume the worst case then. This usually leads to cell variant "
+  "formation which is not always desired and blows up the hierarchy.\n"
+  "\n"
+  "Here is some example that filters texts with a given string length:"
+  "\n"
+  "@code\n"
+  "class TextStringLengthFilter < RBA::TextFilter\n"
+  "\n"
+  "  # Constructor\n"
+  "  def initialize(string_length)\n"
+  "    self.is_isotropic_and_scale_invariant   # orientation and scale do not matter\n"
+  "    @string_length = string_length\n"
+  "  end\n"
+  "  \n"
+  "  # Select texts with given string length\n"
+  "  def selected(text)\n"
+  "    return text.string.size == @string_length\n"
+  "  end\n"
+  "\n"
+  "end\n"
+  "\n"
+  "texts = ... # some Texts object\n"
+  "with_length_3 = edges.filtered(TextStringLengthFilter::new(3))\n"
+  "@/code\n"
+  "\n"
+  "This class has been introduced in version 0.29.\n"
+);
+
+// ---------------------------------------------------------------------------------
+//  TextProcessor binding
+
+Class<shape_processor_impl<db::TextProcessorBase> > decl_TextProcessor ("db", "TextOperator",
+  shape_processor_impl<db::TextProcessorBase>::method_decls (false),
+  "@brief A generic text operator\n"
+  "\n"
+  "Text processors are an efficient way to process texts from an text collection. To apply a processor, derive your own "
+  "operator class and pass an instance to the \\Texts#processed or \\Texts#process method.\n"
+  "\n"
+  "Conceptually, these methods take each text from the edge pair collection and present it to the operator's 'process' method.\n"
+  "The result of this call is a list of zero to many output texts derived from the input text.\n"
+  "The output text collection is the sum over all these individual results.\n"
+  "\n"
+  "The magic happens when deep mode text collections are involved. In that case, the processor will use as few calls as possible "
+  "and exploit the hierarchical compression if possible. It needs to know however, how the operator behaves. You "
+  "need to configure the operator by calling \\is_isotropic, \\is_scale_invariant or \\is_isotropic_and_scale_invariant "
+  "before using it.\n"
+  "\n"
+  "You can skip this step, but the processor algorithm will assume the worst case then. This usually leads to cell variant "
+  "formation which is not always desired and blows up the hierarchy.\n"
+  "\n"
+  "Here is some example that replaces the text string:"
+  "\n"
+  "@code\n"
+  "class ReplaceTextString < RBA::TextOperator\n"
+  "\n"
+  "  # Constructor\n"
+  "  def initialize\n"
+  "    self.is_isotropic_and_scale_invariant   # orientation and scale do not matter\n"
+  "  end\n"
+  "  \n"
+  "  # Replaces the string by a number representing the string length\n"
+  "  def process(text)\n"
+  "    new_text = text.dup   # need a copy as we cannot modify the text passed\n"
+  "    new_text.string = text.string.size.to_s\n"
+  "    return [ new_text ]\n"
+  "  end\n"
+  "\n"
+  "end\n"
+  "\n"
+  "texts = ... # some Texts object\n"
+  "modified = texts.processed(ReplaceTextString::new)\n"
+  "@/code\n"
+  "\n"
+  "This class has been introduced in version 0.29.\n"
+);
+
+Class<shape_processor_impl<db::TextToPolygonProcessorBase> > decl_TextToPolygonProcessor ("db", "TextToPolygonOperator",
+  shape_processor_impl<db::TextToPolygonProcessorBase>::method_decls (false),
+  "@brief A generic text-to-polygon operator\n"
+  "\n"
+  "Text processors are an efficient way to process texts from an text collection. To apply a processor, derive your own "
+  "operator class and pass an instance to the \\Texts#processed method.\n"
+  "\n"
+  "Conceptually, these methods take each text from the text collection and present it to the operator's 'process' method.\n"
+  "The result of this call is a list of zero to many output polygons derived from the input text.\n"
+  "The output region is the sum over all these individual results.\n"
+  "\n"
+  "The magic happens when deep mode text collections are involved. In that case, the processor will use as few calls as possible "
+  "and exploit the hierarchical compression if possible. It needs to know however, how the operator behaves. You "
+  "need to configure the operator by calling \\is_isotropic, \\is_scale_invariant or \\is_isotropic_and_scale_invariant "
+  "before using it.\n"
+  "\n"
+  "You can skip this step, but the processor algorithm will assume the worst case then. This usually leads to cell variant "
+  "formation which is not always desired and blows up the hierarchy.\n"
+  "\n"
+  "For a basic example see the \\TextOperator class, with the exception that this incarnation delivers polygons.\n"
+  "\n"
+  "This class has been introduced in version 0.29.\n"
+);
+
+// ---------------------------------------------------------------------------------
+//  Texts binding
+
 static db::Texts *new_v ()
 {
   return new db::Texts ();
@@ -152,6 +306,33 @@ static size_t id (const db::Texts *t)
   return tl::id_of (t->delegate ());
 }
 
+static db::Texts filtered (const db::Texts *r, const TextFilterImpl *f)
+{
+  return r->filtered (*f);
+}
+
+static void filter (db::Texts *r, const TextFilterImpl *f)
+{
+  r->filter (*f);
+}
+
+static db::Texts processed_tt (const db::Texts *r, const shape_processor_impl<db::TextProcessorBase> *f)
+{
+  return r->processed (*f);
+}
+
+static void process_tt (db::Texts *r, const shape_processor_impl<db::TextProcessorBase> *f)
+{
+  r->process (*f);
+}
+
+static db::Region processed_tp (const db::Texts *r, const shape_processor_impl<db::TextToPolygonProcessorBase> *f)
+{
+  db::Region out;
+  r->processed (out, *f);
+  return out;
+}
+
 static db::Texts with_text (const db::Texts *r, const std::string &text, bool inverse)
 {
   db::TextStringFilter f (text, inverse);
@@ -255,6 +436,12 @@ Class<db::Texts> decl_Texts (decl_dbShapeCollection, "db", "Texts",
     "r = RBA::Texts::new(layout.begin_shapes(cell, layer), RBA::ICplxTrans::new(layout.dbu / dbu))\n"
     "@/code\n"
   ) +
+  method ("write", &db::Texts::write, gsi::arg ("filename"),
+    "@brief Writes the region to a file\n"
+    "This method is provided for debugging purposes. It writes the object to a flat layer 0/0 in a single top cell.\n"
+    "\n"
+    "This method has been introduced in version 0.29."
+  ) +
   method ("insert_into", &db::Texts::insert_into, gsi::arg ("layout"), gsi::arg ("cell_index"), gsi::arg ("layer"),
     "@brief Inserts this texts into the given layout, below the given cell and into the given layer.\n"
     "If the text collection is a hierarchical one, a suitable hierarchy will be built below the top cell or "
@@ -401,6 +588,36 @@ Class<db::Texts> decl_Texts (decl_dbShapeCollection, "db", "Texts",
     "@brief Converts the edge pairs to polygons\n"
     "This method creates polygons from the texts. This is equivalent to calling \\extents."
   ) +
+  method_ext ("filter", &filter, gsi::arg ("filter"),
+    "@brief Applies a generic filter in place (replacing the texts from the Texts collection)\n"
+    "See \\TextFilter for a description of this feature.\n"
+    "\n"
+    "This method has been introduced in version 0.29.\n"
+  ) +
+  method_ext ("filtered", &filtered, gsi::arg ("filtered"),
+    "@brief Applies a generic filter and returns a filtered copy\n"
+    "See \\TextFilter for a description of this feature.\n"
+    "\n"
+    "This method has been introduced in version 0.29.\n"
+  ) +
+  method_ext ("process", &process_tt, gsi::arg ("process"),
+    "@brief Applies a generic text processor in place (replacing the texts from the text collection)\n"
+    "See \\TextProcessor for a description of this feature.\n"
+    "\n"
+    "This method has been introduced in version 0.29.\n"
+  ) +
+  method_ext ("processed", &processed_tt, gsi::arg ("processed"),
+    "@brief Applies a generic text processor and returns a processed copy\n"
+    "See \\TextProcessor for a description of this feature.\n"
+    "\n"
+    "This method has been introduced in version 0.29.\n"
+  ) +
+  method_ext ("processed", &processed_tp, gsi::arg ("processed"),
+    "@brief Applies a generic text-to-polygon processor and returns a region with the results\n"
+    "See \\TextToPolygonProcessor for a description of this feature.\n"
+    "\n"
+    "This method has been introduced in version 0.29.\n"
+  ) +
   method_ext ("with_text", with_text, gsi::arg ("text"), gsi::arg ("inverse"),
     "@brief Filter the text by text string\n"
     "If \"inverse\" is false, this method returns the texts with the given string.\n"
