@@ -133,7 +133,7 @@ TEST(3_Edge2EdgeBooleans)
   db::Layout ly;
   {
     std::string fn (tl::testdata ());
-    fn += "/algo/deep_region_l1.gds";
+    fn += "/algo/deep_edges_l1.gds";
     tl::InputStream stream (fn);
     db::Reader reader (stream);
     reader.read (ly);
@@ -145,15 +145,23 @@ TEST(3_Edge2EdgeBooleans)
   db::DeepShapeStore dss;
 
   unsigned int l2 = ly.get_layer (db::LayerProperties (2, 0));
+  unsigned int l21 = ly.get_layer (db::LayerProperties (2, 1));
   unsigned int l3 = ly.get_layer (db::LayerProperties (3, 0));
+  unsigned int lempty = ly.insert_layer ();
 
   db::Region r2 (db::RecursiveShapeIterator (ly, top_cell, l2), dss);
+  db::Region r21 (db::RecursiveShapeIterator (ly, top_cell, l21), dss);
   db::Region r3 (db::RecursiveShapeIterator (ly, top_cell, l3), dss);
   db::Region r2and3 = r2 & r3;
 
   db::Edges e2 = r2.edges ();
+  db::Edges e21 = r21.edges ();
   db::Edges e3 = r3.edges ();
+  db::Edges e3copy = r3.edges ();
   db::Edges e2and3 = r2and3.edges ();
+  db::Edges eempty (db::RecursiveShapeIterator (ly, top_cell, lempty), dss);
+  db::Edges edots = e2and3.processed (db::EdgeSegmentSelector (-1, 0, 0));
+  db::Edges edotscopy = e2and3.processed (db::EdgeSegmentSelector (-1, 0, 0));
 
   db::Layout target;
   unsigned int target_top_cell_index = target.add_cell (ly.cell_name (top_cell_index));
@@ -162,11 +170,70 @@ TEST(3_Edge2EdgeBooleans)
   target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (3, 0)), r3);
   target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (10, 0)), e3);
   target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (11, 0)), e2and3);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (12, 0)), edots);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (13, 0)), edots.merged ());
+
   target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (20, 0)), e3 & e2and3);
-  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (21, 0)), e3 - e2and3);
-  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (22, 0)), e3 ^ e2and3);
-  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (23, 0)), e3.intersections(e2and3));
-  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (24, 0)), e3.intersections(e2));
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (21, 0)), e3 & edots);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (22, 0)), e3 & eempty);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (23, 0)), e3 & e3copy);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (24, 0)), eempty & e2and3);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (25, 0)), edots & edotscopy);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (26, 0)), edots & e2);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (27, 0)), e21 & edots);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (28, 0)), edots & e21);
+
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (30, 0)), e3 - e2and3);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (31, 0)), e3 - edots);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (32, 0)), e3 - eempty);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (33, 0)), e3 - e3copy);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (34, 0)), eempty - e2and3);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (35, 0)), edots - edotscopy);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (36, 0)), edots - e2);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (37, 0)), e21 - edots);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (38, 0)), edots - e21);
+
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (40, 0)), e3 ^ e2and3);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (41, 0)), e3 ^ edots);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (42, 0)), e3 ^ eempty);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (43, 0)), e3 ^ e3copy);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (44, 0)), eempty ^ e2and3);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (45, 0)), edots ^ edotscopy);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (46, 0)), edots ^ e2);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (47, 0)), e21 ^ edots);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (48, 0)), edots ^ e21);
+
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (50, 0)), e3.andnot(e2and3).first);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (51, 0)), e3.andnot(edots).first);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (52, 0)), e3.andnot(eempty).first);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (53, 0)), e3.andnot(e3copy).first);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (54, 0)), eempty.andnot(e2and3).first);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (55, 0)), edots.andnot(edotscopy).first);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (56, 0)), edots.andnot(e2).first);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (57, 0)), e21.andnot(edots).first);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (58, 0)), edots.andnot(e21).first);
+
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (60, 0)), e3.andnot(e2and3).second);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (61, 0)), e3.andnot(edots).second);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (62, 0)), e3.andnot(eempty).second);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (63, 0)), e3.andnot(e3copy).second);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (64, 0)), eempty.andnot(e2and3).second);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (65, 0)), edots.andnot(edotscopy).second);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (66, 0)), edots.andnot(e2).second);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (67, 0)), e21.andnot(edots).second);
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (68, 0)), edots.andnot(e21).second);
+
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (70, 0)), e3.intersections(e2and3));
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (71, 0)), e3.intersections(edots));
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (72, 0)), e3.intersections(eempty));
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (73, 0)), e3.intersections(e3copy));
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (74, 0)), eempty.intersections(e2and3));
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (75, 0)), edots.intersections(edotscopy));
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (76, 0)), edots.intersections(e2));
+  //  test, whether dots are not merged
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (77, 0)), edots.intersections(e2).select_interacting(e2));
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (78, 0)), e21.intersections(edots));
+  target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (79, 0)), edots.intersections(e21));
 
   CHECKPOINT();
   db::compare_layouts (_this, target, tl::testdata () + "/algo/deep_edges_au3.gds");
@@ -260,8 +327,8 @@ TEST(5_Filters)
 
     target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (2, 0)), r2);
 
-    db::EdgeOrientationFilter eof1 (0, true, 1, true, false);
-    db::EdgeOrientationFilter eof2 (0, true, 1, true, true);
+    db::EdgeOrientationFilter eof1 (0, true, 1, true, false, false);
+    db::EdgeOrientationFilter eof2 (0, true, 1, true, true, false);
 
     target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (10, 0)), e2.filtered (eof1));
     target.insert (target_top_cell_index, target.get_layer (db::LayerProperties (11, 0)), e2.filtered (eof2));
@@ -1293,6 +1360,201 @@ TEST(20_in_and_out)
   db::compare_layouts (_this, target, tl::testdata () + "/algo/deep_edges_au20.gds");
 }
 
+TEST(21_EdgeMergeWithDots)
+{
+  db::Edges e;
+  e.insert (db::Edge (db::Point(0, 0), db::Point (100, 0)));
+  e.insert (db::Edge (db::Point(110, 0), db::Point (110, 0)));
+
+  db::Edges ee = e;
+  ee.insert (db::Edge (db::Point(100, 0), db::Point (110, 0)));
+
+  db::Edges eee;
+  eee.insert (db::Edge (db::Point(110, 0), db::Point (110, 0)));
+  eee.insert (db::Edge (db::Point(110, 0), db::Point (110, 0)));
+
+  //  make deep
+
+  db::DeepShapeStore dss;
+
+  db::Layout ly;
+  ly.add_cell ("TOP");
+  unsigned int l1 = ly.insert_layer ();
+  unsigned int l2 = ly.insert_layer ();
+  unsigned int l3 = ly.insert_layer ();
+
+  db::cell_index_type top_cell_index = *ly.begin_top_down ();
+  db::Cell &top_cell = ly.cell (top_cell_index);
+
+  e.insert_into (&ly, top_cell.cell_index (), l1);
+  e = db::Edges (db::RecursiveShapeIterator (ly, top_cell, l1), dss);
+
+  ee.insert_into (&ly, top_cell.cell_index (), l2);
+  ee = db::Edges (db::RecursiveShapeIterator (ly, top_cell, l2), dss);
+
+  eee.insert_into (&ly, top_cell.cell_index (), l3);
+  eee = db::Edges (db::RecursiveShapeIterator (ly, top_cell, l3), dss);
+
+  EXPECT_EQ (e.merged ().to_string (), "(0,0;100,0);(110,0;110,0)");
+  //  dots do not participate in merge
+  EXPECT_EQ (ee.merged ().to_string (), "(0,0;110,0)");
+  //  dots do not participate in merge
+  EXPECT_EQ (eee.merged ().to_string (), "(110,0;110,0)");
+}
+
+TEST(22_InteractingWithCount)
+{
+  db::Edges e;
+  e.insert (db::Edge (db::Point (0, 0), db::Point (100, 0)));
+  e.insert (db::Edge (db::Point (100, 0), db::Point (200, 0)));
+  e.insert (db::Edge (db::Point (0, 10), db::Point (200, 10)));
+  e.insert (db::Edge (db::Point (0, 20), db::Point (200, 20)));
+  e.insert (db::Edge (db::Point (0, 30), db::Point (200, 30)));
+
+  db::Edges e2;
+  e2.insert (db::Edge (db::Point (100, 0), db::Point (100, 10)));
+  e2.insert (db::Edge (db::Point (100, 0), db::Point (100, 30)));
+  e2.insert (db::Edge (db::Point (110, 10), db::Point (110, 30)));
+  e2.merge ();
+  e2.insert (db::Edge (db::Point (120, 20), db::Point (120, 20)));
+  e2.insert (db::Edge (db::Point (130, 30), db::Point (130, 30)));
+  e2.set_merged_semantics (false);
+
+  db::Region r2;
+  r2.insert (db::Box (db::Point (99, 0), db::Point (101, 10)));
+  r2.insert (db::Box (db::Point (99, 0), db::Point (101, 30)));
+  r2.insert (db::Box (db::Point (109, 10), db::Point (111, 30)));
+  r2.insert (db::Box (db::Point (119, 19), db::Point (121, 21)));
+  r2.insert (db::Box (db::Point (129, 29), db::Point (131, 31)));
+
+  //  make deep
+
+  db::DeepShapeStore dss;
+
+  db::Layout ly;
+  ly.add_cell ("TOP");
+  unsigned int l1 = ly.insert_layer ();
+  unsigned int l2 = ly.insert_layer ();
+  unsigned int l3 = ly.insert_layer ();
+
+  db::cell_index_type top_cell_index = *ly.begin_top_down ();
+  db::Cell &top_cell = ly.cell (top_cell_index);
+
+  e.insert_into (&ly, top_cell.cell_index (), l1);
+  e = db::Edges (db::RecursiveShapeIterator (ly, top_cell, l1), dss);
+
+  e2.insert_into (&ly, top_cell.cell_index (), l2);
+  e2 = db::Edges (db::RecursiveShapeIterator (ly, top_cell, l2), dss);
+  //  because it has dots
+  e2.set_merged_semantics (false);
+
+  r2.insert_into (&ly, top_cell.cell_index (), l3);
+  r2 = db::Region (db::RecursiveShapeIterator (ly, top_cell, l3), dss);
+
+
+  db::Edges edup;
+
+  EXPECT_EQ (db::compare (e.selected_interacting (e2), "(0,0;200,0);(0,10;200,10);(0,20;200,20);(0,30;200,30)"), true);
+  EXPECT_EQ (db::compare (e.selected_interacting (e2, size_t (2)), "(0,10;200,10);(0,20;200,20);(0,30;200,30)"), true);
+  EXPECT_EQ (db::compare (e.selected_interacting (e2, size_t (2), size_t(2)), "(0,10;200,10)"), true);
+  EXPECT_EQ (db::compare (e.selected_interacting (e2, size_t (2), size_t(3)), "(0,10;200,10);(0,20;200,20);(0,30;200,30)"), true);
+  EXPECT_EQ (db::compare (e.selected_interacting (e2, size_t (3)), "(0,20;200,20);(0,30;200,30)"), true);
+  EXPECT_EQ (db::compare (e.selected_interacting (e2, size_t (4)), ""), true);
+
+  edup = e;
+  edup.select_interacting (e2, size_t (2), size_t(3));
+  EXPECT_EQ (db::compare (edup, "(0,10;200,10);(0,20;200,20);(0,30;200,30)"), true);
+
+  EXPECT_EQ (db::compare (e.selected_not_interacting (e2), ""), true);
+  EXPECT_EQ (db::compare (e.selected_not_interacting (e2, size_t (2)), "(0,0;200,0)"), true);
+  EXPECT_EQ (db::compare (e.selected_not_interacting (e2, size_t (2), size_t(2)), "(0,0;200,0);(0,20;200,20);(0,30;200,30)"), true);
+  EXPECT_EQ (db::compare (e.selected_not_interacting (e2, size_t (2), size_t(3)), "(0,0;200,0)"), true);
+  EXPECT_EQ (db::compare (e.selected_not_interacting (e2, size_t (3)), "(0,0;200,0);(0,10;200,10)"), true);
+  EXPECT_EQ (db::compare (e.selected_not_interacting (e2, size_t (4)), "(0,0;200,0);(0,10;200,10);(0,20;200,20);(0,30;200,30)"), true);
+
+  edup = e;
+  edup.select_not_interacting (e2, size_t (2), size_t(3));
+  EXPECT_EQ (db::compare (edup, "(0,0;200,0)"), true);
+
+  EXPECT_EQ (db::compare (e.selected_interacting_differential (e2, size_t (2), size_t(3)).first, "(0,10;200,10);(0,20;200,20);(0,30;200,30)"), true);
+  EXPECT_EQ (db::compare (e.selected_interacting_differential (e2, size_t (2), size_t(3)).second, "(0,0;200,0)"), true);
+
+  EXPECT_EQ (db::compare (e.selected_interacting (r2), "(0,0;200,0);(0,10;200,10);(0,20;200,20);(0,30;200,30)"), true);
+  EXPECT_EQ (db::compare (e.selected_interacting (r2, size_t (2)), "(0,10;200,10);(0,20;200,20);(0,30;200,30)"), true);
+  EXPECT_EQ (db::compare (e.selected_interacting (r2, size_t (2), size_t(2)), "(0,10;200,10)"), true);
+  EXPECT_EQ (db::compare (e.selected_interacting (r2, size_t (2), size_t(3)), "(0,10;200,10);(0,20;200,20);(0,30;200,30)"), true);
+  EXPECT_EQ (db::compare (e.selected_interacting (r2, size_t (3)), "(0,20;200,20);(0,30;200,30)"), true);
+  EXPECT_EQ (db::compare (e.selected_interacting (r2, size_t (4)), ""), true);
+
+  edup = e;
+  edup.select_interacting (r2, size_t (2), size_t(3));
+  EXPECT_EQ (db::compare (edup, "(0,10;200,10);(0,20;200,20);(0,30;200,30)"), true);
+
+  EXPECT_EQ (db::compare (e.selected_not_interacting (r2), ""), true);
+  EXPECT_EQ (db::compare (e.selected_not_interacting (r2, size_t (2)), "(0,0;200,0)"), true);
+  EXPECT_EQ (db::compare (e.selected_not_interacting (r2, size_t (2), size_t(2)), "(0,0;200,0);(0,20;200,20);(0,30;200,30)"), true);
+  EXPECT_EQ (db::compare (e.selected_not_interacting (r2, size_t (2), size_t(3)), "(0,0;200,0)"), true);
+  EXPECT_EQ (db::compare (e.selected_not_interacting (r2, size_t (3)), "(0,0;200,0);(0,10;200,10)"), true);
+  EXPECT_EQ (db::compare (e.selected_not_interacting (r2, size_t (4)), "(0,0;200,0);(0,10;200,10);(0,20;200,20);(0,30;200,30)"), true);
+
+  edup = e;
+  edup.select_not_interacting (r2, size_t (2), size_t(3));
+  EXPECT_EQ (db::compare (edup, "(0,0;200,0)"), true);
+
+  EXPECT_EQ (db::compare (e.selected_interacting_differential (r2, size_t (2), size_t(3)).first, "(0,10;200,10);(0,20;200,20);(0,30;200,30)"), true);
+  EXPECT_EQ (db::compare (e.selected_interacting_differential (r2, size_t (2), size_t(3)).second, "(0,0;200,0)"), true);
+}
+
+TEST(23_SameInputs)
+{
+  db::Layout ly;
+  {
+    std::string fn (tl::testdata ());
+    fn += "/algo/deep_region_l1.gds";
+    tl::InputStream stream (fn);
+    db::Reader reader (stream);
+    reader.read (ly);
+  }
+
+  db::cell_index_type top_cell_index = *ly.begin_top_down ();
+  db::Cell &top_cell = ly.cell (top_cell_index);
+
+  db::DeepShapeStore dss;
+
+  unsigned int l2 = ly.get_layer (db::LayerProperties (2, 0));
+  db::Edges e2 = db::Edges ((db::Region (db::RecursiveShapeIterator (ly, top_cell, l2), dss)).edges ());
+
+  EXPECT_EQ ((e2 & e2).to_string (), e2.to_string ());
+  EXPECT_EQ ((e2 - e2).to_string (), "");
+  EXPECT_EQ (e2.andnot (e2).first.to_string (), e2.to_string ());
+  EXPECT_EQ (e2.andnot (e2).second.to_string (), "");
+  EXPECT_EQ ((e2 | e2).to_string (), e2.to_string ());
+  EXPECT_EQ ((e2 ^ e2).to_string (), "");
+  EXPECT_EQ (e2.in (e2).to_string (), e2.to_string ());
+  EXPECT_EQ (e2.in (e2, true).to_string (), "");
+  EXPECT_EQ (e2.in_and_out (e2).first.to_string (), e2.to_string ());
+  EXPECT_EQ (e2.in_and_out (e2).second.to_string (), "");
+  EXPECT_EQ (e2.selected_interacting (e2).to_string (), e2.to_string ());
+  EXPECT_EQ (e2.selected_not_interacting (e2).to_string (), "");
+  EXPECT_EQ (e2.selected_interacting_differential (e2).first.to_string (), e2.to_string ());
+  EXPECT_EQ (e2.selected_interacting_differential (e2).second.to_string (), "");
+  EXPECT_EQ ((e2.selected_interacting (e2, (size_t) 1, (size_t) 3) ^ e2).to_string (), "");
+  EXPECT_EQ ((e2.selected_interacting_differential (e2, (size_t) 1, (size_t) 3).first ^ e2).to_string (), "");
+  EXPECT_EQ (e2.selected_interacting_differential (e2, (size_t) 1, (size_t) 3).second.to_string (), "");
+  EXPECT_EQ (e2.selected_interacting (e2, (size_t) 4).to_string (), "");
+  EXPECT_EQ (e2.selected_interacting_differential (e2, (size_t) 4).first.to_string (), "");
+  EXPECT_EQ ((e2.selected_interacting_differential (e2, (size_t) 4).second ^ e2).to_string (), "");
+  EXPECT_EQ (e2.selected_inside (e2).to_string (), e2.to_string ());
+  EXPECT_EQ (e2.selected_not_inside (e2).to_string (), "");
+  EXPECT_EQ (e2.selected_inside_differential (e2).first.to_string (), e2.to_string ());
+  EXPECT_EQ (e2.selected_inside_differential (e2).second.to_string (), "");
+  EXPECT_EQ (e2.selected_outside (e2).to_string (), "");
+  EXPECT_EQ (e2.selected_not_outside (e2).to_string (), e2.to_string ());
+  EXPECT_EQ (e2.selected_outside_differential (e2).first.to_string (), "");
+  EXPECT_EQ (e2.selected_outside_differential (e2).second.to_string (), e2.to_string ());
+  EXPECT_EQ (e2.pull_interacting (e2).to_string (), e2.to_string ());
+}
+
 TEST(deep_edges_and_cheats)
 {
   db::Layout ly;
