@@ -26,6 +26,10 @@
 #include "dbLayoutDiff.h"
 #include "tlString.h"
 #include "tlUnitTest.h"
+#include "tlFileUtils.h"
+#include "tlStream.h"
+#include "dbReader.h"
+#include "dbWriter.h"
 
 #include <vector>
 
@@ -1554,3 +1558,225 @@ TEST(11_LayoutIsWeakPointer)
   x = collect(i1, *g);
   EXPECT_EQ (x, "");
 }
+
+TEST(12_ForMerged)
+{
+  std::unique_ptr<db::Layout> g (new db::Layout ());
+  g->insert_layer (0);
+  g->insert_layer (1);
+  db::Cell &c0 (g->cell (g->add_cell ()));
+  db::Cell &c1 (g->cell (g->add_cell ()));
+  db::Cell &c2 (g->cell (g->add_cell ()));
+  db::Cell &c3 (g->cell (g->add_cell ()));
+
+  db::Box b (0, 100, 1000, 1200);
+  c0.shapes (0).insert (db::Box (0, 0, 3000, 2000));
+  c1.shapes (0).insert (b);
+  c2.shapes (0).insert (b);
+  c3.shapes (0).insert (b);
+
+  db::Trans tt;
+  c0.insert (db::CellInstArray (db::CellInst (c1.cell_index ()), tt));
+  c0.insert (db::CellInstArray (db::CellInst (c2.cell_index ()), db::Trans (db::Vector (100, -100))));
+  c0.insert (db::CellInstArray (db::CellInst (c3.cell_index ()), db::Trans (1)));
+  c2.insert (db::CellInstArray (db::CellInst (c3.cell_index ()), db::Trans (db::Vector (1100, 0))));
+
+  std::string x;
+
+  db::RecursiveShapeIterator i1 (*g, c0, 0);
+  x = collect(i1, *g);
+  EXPECT_EQ (x, "[$1](0,0;3000,2000)/[$2](0,100;1000,1200)/[$3](100,0;1100,1100)/[$4](1200,0;2200,1100)/[$4](-1200,0;-100,1000)");
+
+  i1.set_for_merged_input (true);
+  x = collect(i1, *g);
+  EXPECT_EQ (x, "[$1](0,0;3000,2000)/[$4](-1200,0;-100,1000)");
+
+  std::vector<unsigned int> lv;
+  lv.push_back (0);
+  i1 = db::RecursiveShapeIterator (*g, c0, lv);
+  x = collect(i1, *g);
+  EXPECT_EQ (x, "[$1](0,0;3000,2000)/[$2](0,100;1000,1200)/[$3](100,0;1100,1100)/[$4](1200,0;2200,1100)/[$4](-1200,0;-100,1000)");
+
+  i1.set_for_merged_input (true);
+  x = collect(i1, *g);
+  EXPECT_EQ (x, "[$1](0,0;3000,2000)/[$4](-1200,0;-100,1000)");
+
+  lv.push_back (1); //  empty, but kills "for merged" optimization
+  i1 = db::RecursiveShapeIterator (*g, c0, lv);
+  x = collect(i1, *g);
+  EXPECT_EQ (x, "[$1](0,0;3000,2000)/[$2](0,100;1000,1200)/[$3](100,0;1100,1100)/[$4](1200,0;2200,1100)/[$4](-1200,0;-100,1000)");
+
+  i1.set_for_merged_input (true);
+  x = collect(i1, *g);
+  //  no longer optimized
+  EXPECT_EQ (x, "[$1](0,0;3000,2000)/[$2](0,100;1000,1200)/[$3](100,0;1100,1100)/[$4](1200,0;2200,1100)/[$4](-1200,0;-100,1000)");
+
+  i1 = db::RecursiveShapeIterator (*g, c0, 0, db::Box (-100, 0, 100, 50));
+  x = collect(i1, *g);
+  EXPECT_EQ (x, "[$1](0,0;3000,2000)/[$3](100,0;1100,1100)/[$4](-1200,0;-100,1000)");
+
+  i1.set_for_merged_input (true);
+  x = collect(i1, *g);
+  EXPECT_EQ (x, "[$1](0,0;3000,2000)/[$4](-1200,0;-100,1000)");
+
+  i1 = db::RecursiveShapeIterator (*g, c0, 0, db::Box (-101, 0, 100, 50));
+  i1.set_overlapping (true);
+  x = collect(i1, *g);
+  EXPECT_EQ (x, "[$1](0,0;3000,2000)/[$4](-1200,0;-100,1000)");
+
+  i1.set_for_merged_input (true);
+  x = collect(i1, *g);
+  EXPECT_EQ (x, "[$1](0,0;3000,2000)/[$4](-1200,0;-100,1000)");
+}
+
+
+TEST(13_ForMergedPerformance)
+{
+  test_is_long_runner ();
+
+  std::string fn (tl::combine_path (tl::testdata_private (), "oasis/caravel.oas.gz"));
+
+  db::Layout ly;
+
+  {
+    tl::InputStream is (fn);
+    db::Reader reader (is);
+    reader.read (ly);
+  }
+
+  unsigned l1 = ly.get_layer (db::LayerProperties (66, 20));
+  unsigned l2 = ly.get_layer (db::LayerProperties (235, 4));
+
+  db::RecursiveShapeIterator si1 (ly, ly.cell (*ly.begin_top_down ()), l1);
+  db::RecursiveShapeIterator si2 (ly, ly.cell (*ly.begin_top_down ()), l2);
+
+  size_t n1_expected_full = db::default_editable_mode () ? 1203072 : 1203078;
+  size_t n2_expected_full = 10;
+
+  {
+    tl::SelfTimer timer ("Standard loop on 66/20");
+    size_t n = 0;
+    while (! si1.at_end ()) {
+      ++si1;
+      ++n;
+    }
+    tl::info << "Counted " << n << " shapes on 66/20";
+    EXPECT_EQ (n, size_t (1218378));
+  }
+
+  {
+    tl::SelfTimer timer ("Standard loop on 235/4");
+    size_t n = 0;
+    while (! si2.at_end ()) {
+      ++si2;
+      ++n;
+    }
+    tl::info << "Counted " << n << " shapes on 235/4";
+    EXPECT_EQ (n, size_t (57462));
+  }
+
+  si1.set_for_merged_input (true);
+  si2.set_for_merged_input (true);
+
+  {
+    tl::SelfTimer timer ("'for_merged' loop on 66/20");
+    size_t n = 0;
+    while (! si1.at_end ()) {
+      ++si1;
+      ++n;
+    }
+    tl::info << "Counted " << n << " shapes on 66/20";
+    EXPECT_EQ (n, size_t (n1_expected_full));
+  }
+
+  {
+    tl::SelfTimer timer ("'for_merged' loop on 235/4");
+    size_t n = 0;
+    while (! si2.at_end ()) {
+      ++si2;
+      ++n;
+    }
+    tl::info << "Counted " << n << " shapes on 235/4";
+    EXPECT_EQ (n, size_t (n2_expected_full));
+  }
+
+  si1.set_for_merged_input (false);
+  si1.set_region (db::Box (0, 0, 1000000, 1000000));
+  si2.set_for_merged_input (false);
+  si2.set_region (db::Box (0, 0, 1000000, 1000000));
+
+  {
+    tl::SelfTimer timer ("Standard loop on 66/20");
+    size_t n = 0;
+    while (! si1.at_end ()) {
+      ++si1;
+      ++n;
+    }
+    tl::info << "Counted " << n << " shapes on 66/20";
+    EXPECT_EQ (n, size_t (218823));
+  }
+
+  {
+    tl::SelfTimer timer ("Standard loop on 235/4");
+    size_t n = 0;
+    while (! si2.at_end ()) {
+      ++si2;
+      ++n;
+    }
+    tl::info << "Counted " << n << " shapes on 235/4";
+    EXPECT_EQ (n, size_t (2578));
+  }
+
+  si1.set_for_merged_input (true);
+  si2.set_for_merged_input (true);
+
+  size_t n1_expected = db::default_editable_mode () ? 218068 : 218069;
+  size_t n2_expected = 2;
+
+  {
+    tl::SelfTimer timer ("'for_merged' loop on 66/20");
+    size_t n = 0;
+    while (! si1.at_end ()) {
+      ++si1;
+      ++n;
+    }
+    tl::info << "Counted " << n << " shapes on 66/20";
+    EXPECT_EQ (n, size_t (n1_expected));
+  }
+
+  {
+    tl::SelfTimer timer ("'for_merged' loop on 235/4");
+    size_t n = 0;
+    while (! si2.at_end ()) {
+      ++si2;
+      ++n;
+    }
+    tl::info << "Counted " << n << " shapes on 235/4";
+    EXPECT_EQ (n, size_t (n2_expected));
+  }
+
+  {
+    tl::SelfTimer timer ("XOR on tile of 66/20");
+    si1.set_for_merged_input (false);
+    db::Region r1 (si1);
+    si1.set_for_merged_input (true);
+    db::Region r2 (si1);
+
+    EXPECT_EQ (r1.count (), size_t (218823));
+    EXPECT_EQ (r2.count (), size_t (n1_expected));
+    EXPECT_EQ ((r1 ^ r2).count (), size_t (0));
+  }
+
+  {
+    tl::SelfTimer timer ("XOR on tile of 235/4");
+    si2.set_for_merged_input (false);
+    db::Region r1 (si2);
+    si2.set_for_merged_input (true);
+    db::Region r2 (si2);
+
+    EXPECT_EQ (r1.count (), size_t (2578));
+    EXPECT_EQ (r2.count (), size_t (n2_expected));
+    EXPECT_EQ ((r1 ^ r2).count (), size_t (0));
+  }
+}
+
