@@ -115,11 +115,29 @@ module DRC
     end
 
     def write
+
       if @file_name
+      
         rdb_file = @engine._make_path(@file_name)
+
+        #  Apply waive DB if possible
+        wdb_file = rdb_file + ".w"
+        if File.exists?(wdb_file)
+          begin
+            wdb = RBA::ReportDatabase::new
+            wdb.load(wdb_file)
+            @engine.info("Applying waive database: #{wdb_file} ..")
+            @rdb.apply(wdb)
+            wdb._destroy
+          rescue
+          end
+        end
+
         @engine.info("Writing report database: #{rdb_file} ..")
         @rdb.save(rdb_file)
+
       end
+
     end
 
     def rdb
@@ -255,6 +273,46 @@ module DRC
       DRCJoinFlag::new(true)
     end
     
+    def convex
+      DRCEdgeMode::new(RBA::EdgeMode::Convex)
+    end
+    
+    def not_convex
+      DRCEdgeMode::new(RBA::EdgeMode::NotConvex)
+    end
+    
+    def concave
+      DRCEdgeMode::new(RBA::EdgeMode::Concave)
+    end
+    
+    def not_concave
+      DRCEdgeMode::new(RBA::EdgeMode::NotConcave)
+    end
+    
+    def step_in
+      DRCEdgeMode::new(RBA::EdgeMode::StepIn)
+    end
+    
+    def not_step_in
+      DRCEdgeMode::new(RBA::EdgeMode::NotStepIn)
+    end
+    
+    def step_out
+      DRCEdgeMode::new(RBA::EdgeMode::StepOut)
+    end
+    
+    def not_step_out
+      DRCEdgeMode::new(RBA::EdgeMode::NotStepOut)
+    end
+    
+    def step
+      DRCEdgeMode::new(RBA::EdgeMode::Step)
+    end
+    
+    def not_step
+      DRCEdgeMode::new(RBA::EdgeMode::NotStep)
+    end
+    
     def padding_zero
       DRCDensityPadding::new(:zero)
     end
@@ -511,6 +569,10 @@ module DRC
       end
     end
 
+    def absolute
+      DRCAbsoluteMode::new(true)
+    end
+    
     def as_dots
       DRCOutputMode::new(:dots)
     end
@@ -2244,12 +2306,36 @@ CODE
     # @synopsis connect(a, b)
     # See \Netter#connect for a description of that function.
  
+    # %DRC%
+    # @name soft_connect
+    # @brief Specifies a soft connection between two layers
+    # @synopsis soft_connect(a, b)
+    # A "soft connection" is made between two layers and
+    # is a directional connection (like an ideal diode).
+    # Soft connections allow detecting if nets are connected
+    # via a high-ohmic substrate or diffusion layer (the
+    # "lower" layer).
+    # "b" is the "lower" and "a" the upper layer.
+    #
+    # See \Netter#connect for a more detailed description of that function.
+ 
     # %DRC%
     # @name connect_global
     # @brief Specifies a connection to a global net
     # @synopsis connect_global(l, name)
     # See \Netter#connect_global for a description of that function.
  
+    # %DRC%
+    # @name soft_connect_global
+    # @brief Specifies a soft connection to a global net
+    # @synopsis soft_connect_global(l, name)
+    # Like \soft_connect, a soft connection is made between 
+    # a layer and a global net (e.g. substrate). The global net
+    # is always the "lower" net of the soft connection.
+    #
+    # See \Netter#soft_connect_global for a more detailed 
+    # description of that function.
+ 
     # %DRC%
     # @name clear_connections
     # @brief Clears all connections stored so far
@@ -2320,14 +2406,18 @@ CODE
       clear_connections
       connect
       connect_global
+      soft_connect
+      soft_connect_global
       connect_implicit
       connect_explicit
       device_scaling
       top_level
       ignore_extraction_errors
       extract_devices
-      l2n_data
       netlist
+      l2n_data
+      _l2n_object
+      _make_soft_connection_diodes
     ).each do |f|
       eval <<"CODE"
         def #{f}(*args)
@@ -3158,7 +3248,33 @@ CODE
         output_rdb = channel.rdb
         output_cell = channel.cell
 
-        cat = output_rdb.create_category(args[0].to_s)
+        categories = args[0]
+        if !categories.is_a?(Array)
+          categories = [ categories.to_s ]
+        end
+
+        cat = nil
+        categories.each do |c|
+          ccat = nil
+          if cat
+            cat.each_sub_category do |i|
+              if i.name == c
+                ccat = i
+                break
+              end
+            end
+          else
+            output_rdb.each_category do |i|
+              if i.name == c
+                ccat = i
+                break
+              end
+            end
+          end
+          cat = ccat ? ccat : output_rdb.create_category(cat, c)
+        end
+        cat ||= output_rdb.create_category("default")
+
         args[1] && cat.description = args[1]
 
         cat.scan_collection(output_cell, RBA::CplxTrans::new(self.dbu), data)
@@ -3265,7 +3381,7 @@ CODE
       else
         output_rdb = RBA::ReportDatabase::new(name)
       end
-      
+
       cn = cellname && cellname.to_s
       cn ||= @def_cell && @def_cell.name
       cn ||= @def_source && @def_source.cell_obj && @def_source.cell_obj.name
@@ -3275,7 +3391,8 @@ CODE
       output_rdb.generator = self._generator
       output_rdb.top_cell_name = cn
       output_rdb.description = description
-
+      output_rdb.original_file = @def_path
+      
       RDBOutputChannel::new(self, output_rdb, output_rdb_index, cn, output_rdb_file)
       
     end
