@@ -25,6 +25,7 @@
 #include "dbTestSupport.h"
 #include "dbNetlist.h"
 #include "dbNetlistSpiceReader.h"
+#include "rdb.h"
 #include "lymMacro.h"
 #include "tlFileUtils.h"
 
@@ -1672,3 +1673,157 @@ TEST(92_issue1594_dual_top)
   CHECKPOINT ();
   compare_netlists (_this, output, au);
 }
+
+TEST(93_withAngle)
+{
+  run_test (_this, "93", false);
+}
+
+TEST(93d_withAngle)
+{
+  run_test (_this, "93", true);
+}
+
+TEST(100_edge_interaction_with_count)
+{
+  run_test (_this, "100", false);
+}
+
+TEST(100d_edge_interaction_with_count)
+{
+  run_test (_this, "100", true);
+}
+
+TEST(101_edge_booleans_with_dots)
+{
+  run_test (_this, "101", false);
+}
+
+TEST(101d_edge_booleans_with_dots)
+{
+  run_test (_this, "101", true);
+}
+
+TEST(102_edge_modes)
+{
+  run_test (_this, "102", false);
+}
+
+TEST(102d_edge_modes)
+{
+  run_test (_this, "102", true);
+}
+
+TEST(110_RDBVariantAssignment)
+{
+  std::string rs = tl::testdata ();
+  rs += "/drc/drcSimpleTests_110.drc";
+
+  //  apart from that it's a variant of 14b ...
+
+  std::string input = tl::testdata ();
+  input += "/drc/drcSimpleTests_110.gds";
+
+  std::string au_report = tl::testdata ();
+  au_report += "/drc/drcSimpleTests_au110.lyrdb";
+
+  std::string report = this->tmp_file ("tmp.lydrc");
+
+  {
+    //  Set some variables
+    lym::Macro config;
+    config.set_text (tl::sprintf (
+        "$drc_force_gc = true\n"
+        "$drc_test_source = '%s'\n"
+        "$drc_test_report = '%s'\n"
+      , input, report)
+    );
+    config.set_interpreter (lym::Macro::Ruby);
+    EXPECT_EQ (config.run (), 0);
+  }
+
+  lym::Macro drc;
+  drc.load_from (rs);
+  EXPECT_EQ (drc.run (), 0);
+
+  compare_text_files (report, au_report);
+}
+
+TEST(111_RDBCategoryHierarchy)
+{
+  std::string rs = tl::testdata ();
+  rs += "/drc/drcSimpleTests_111.drc";
+
+  //  apart from that it's a variant of 14b ...
+
+  std::string input = tl::testdata ();
+  input += "/drc/drcSimpleTests_111.gds";
+
+  std::string au_report = tl::testdata ();
+  au_report += "/drc/drcSimpleTests_au111.lyrdb";
+
+  std::string report = this->tmp_file ("tmp.lydrc");
+
+  {
+    //  Set some variables
+    lym::Macro config;
+    config.set_text (tl::sprintf (
+        "$drc_force_gc = true\n"
+        "$drc_test_source = '%s'\n"
+        "$drc_test_report = '%s'\n"
+      , input, report)
+    );
+    config.set_interpreter (lym::Macro::Ruby);
+    EXPECT_EQ (config.run (), 0);
+  }
+
+  lym::Macro drc;
+  drc.load_from (rs);
+  EXPECT_EQ (drc.run (), 0);
+
+  compare_text_files (report, au_report);
+}
+
+TEST(112_Waiving)
+{
+  std::string rs = tl::testdata ();
+  rs += "/drc/drcSimpleTests_112.drc";
+
+  //  apart from that it's a variant of 14b ...
+
+  std::string input = tl::testdata ();
+  input += "/drc/drcSimpleTests_112.gds";
+
+  std::string au_report = tl::testdata ();
+  au_report += "/drc/drcSimpleTests_au112.lyrdb";
+
+  std::string report = this->tmp_file ("tmp.lydrc");
+
+  {
+    //  Set some variables
+    lym::Macro config;
+    config.set_text (tl::sprintf (
+        "$drc_force_gc = true\n"
+        "$drc_test_source = '%s'\n"
+        "$drc_test_report = '%s'\n"
+      , input, report)
+    );
+    config.set_interpreter (lym::Macro::Ruby);
+    EXPECT_EQ (config.run (), 0);
+  }
+
+  //  prepare a waiver db
+  {
+    std::string report_w = this->tmp_file ("tmp.lydrc.w");
+    rdb::Database rdb_w;
+    rdb_w.load (au_report + ".w");
+    rdb_w.write (report_w);
+  }
+
+  lym::Macro drc;
+  drc.load_from (rs);
+  EXPECT_EQ (drc.run (), 0);
+
+  compare_text_files (report, au_report);
+}
+
