@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -72,7 +72,8 @@ Service::Service (db::Manager *manager, lay::LayoutViewBase *view, db::ShapeIter
     m_flags (flags),
     m_move_sel (false), m_moving (false),
     m_connect_ac (lay::AC_Any), m_move_ac (lay::AC_Any), m_alt_ac (lay::AC_Global),
-    m_snap_to_objects (false),
+    m_snap_to_objects (true),
+    m_snap_objects_to_grid (true),
     m_top_level_sel (false), m_show_shapes_of_instances (true), m_max_shapes_of_instances (1000),
     m_hier_copy_mode (-1),
     m_indicate_secondary_selection (false),
@@ -94,6 +95,7 @@ Service::Service (db::Manager *manager, lay::LayoutViewBase *view)
     m_move_sel (false), m_moving (false),
     m_connect_ac (lay::AC_Any), m_move_ac (lay::AC_Any), m_alt_ac (lay::AC_Global),
     m_snap_to_objects (true),
+    m_snap_objects_to_grid (true),
     m_top_level_sel (false), m_show_shapes_of_instances (true), m_max_shapes_of_instances (1000),
     m_hier_copy_mode (-1),
     m_indicate_secondary_selection (false),
@@ -148,6 +150,123 @@ Service::snap (db::DPoint p) const
   return p;
 }
 
+void
+Service::update_vector_snapped_point (const db::DPoint &pt, db::DVector &vr, bool &result_set) const
+{
+  db::DVector v = snap (pt) - pt;
+
+  if (! result_set || v.length () < vr.length ()) {
+    result_set = true;
+    vr = v;
+  }
+}
+
+void
+Service::update_vector_snapped_marker (const lay::ShapeMarker *sm, const db::DTrans &trans, db::DVector &vr, bool &result_set, size_t &count) const
+{
+  const db::Shape &shape = sm->shape ();
+  db::CplxTrans tr = db::DCplxTrans (trans) * db::DCplxTrans (-sm->trans ().disp ()) * sm->trans ();
+
+  if (shape.is_text ()) {
+
+    update_vector_snapped_point (tr * shape.bbox ().center (), vr, result_set);
+    --count;
+
+  } else if (shape.is_point ()) {
+
+    update_vector_snapped_point (tr * shape.point (), vr, result_set);
+    --count;
+
+  } else if (shape.is_edge ()) {
+
+    update_vector_snapped_point (tr * shape.edge ().p1 (), vr, result_set);
+    --count;
+    if (count > 0) {
+      update_vector_snapped_point (tr * shape.edge ().p2 (), vr, result_set);
+      --count;
+    }
+
+  } else if (shape.is_path ()) {
+
+    for (auto pt = shape.begin_point (); pt != shape.end_point () && count > 0; ++pt) {
+      update_vector_snapped_point (tr * *pt, vr, result_set);
+      --count;
+    }
+
+  } else if (shape.is_box ()) {
+
+    db::Box box = shape.bbox ();
+    for (unsigned int c = 0; c < 4 && count > 0; ++c) {
+      db::Point pt = db::Point ((c & 1) != 0 ? box.left () : box.right (), (c & 2) != 0 ? box.bottom () : box.top ());
+      update_vector_snapped_point (tr * pt, vr, result_set);
+      --count;
+    }
+
+  } else if (shape.is_polygon ()) {
+
+    for (auto pt = shape.begin_hull (); pt != shape.end_hull () && count > 0; ++pt) {
+      update_vector_snapped_point (tr * *pt, vr, result_set);
+      --count;
+    }
+
+    for (unsigned int h = 0; h < shape.holes () && count > 0; ++h) {
+      for (auto pt = shape.begin_hole (h); pt != shape.end_hole (h) && count > 0; ++pt) {
+        update_vector_snapped_point (tr * *pt, vr, result_set);
+        --count;
+      }
+    }
+
+  }
+}
+
+void
+Service::update_vector_snapped_marker (const lay::InstanceMarker *im, const db::DTrans &trans, db::DVector &vr, bool &result_set, size_t &count) const
+{
+  const db::Instance &instance = im->instance ();
+  db::CplxTrans tr = db::DCplxTrans (trans) * db::DCplxTrans (-im->trans ().disp ()) * im->trans ();
+
+  update_vector_snapped_point (tr * (instance.complex_trans () * db::Point ()), vr, result_set);
+  --count;
+}
+
+db::DVector
+Service::snap_marker_to_grid (const db::DVector &v, bool &snapped) const
+{
+  if (! m_snap_objects_to_grid) {
+    return v;
+  }
+
+  snapped = false;
+  db::DVector vr;
+
+  //  max. 10000 checks
+  size_t count = 10000;
+
+  db::DVector snapped_to (1.0, 1.0);
+  db::DVector vv = lay::snap_angle (v, move_ac (), &snapped_to);
+
+  db::DTrans tt = db::DTrans (vv);
+
+  for (auto m = m_markers.begin (); m != m_markers.end () && count > 0; ++m) {
+
+    const lay::ShapeMarker *sm = dynamic_cast<const lay::ShapeMarker *> (*m);
+    const lay::InstanceMarker *im = dynamic_cast<const lay::InstanceMarker *> (*m);
+    if (sm) {
+      update_vector_snapped_marker (sm, tt, vr, snapped, count);
+    } else if (im) {
+      update_vector_snapped_marker (im, tt, vr, snapped, count);
+    }
+
+  }
+
+  if (snapped) {
+    vr += vv;
+    return db::DVector (vr.x () * fabs (snapped_to.x ()), vr.y () * fabs (snapped_to.y ()));
+  } else {
+    return db::DVector ();
+  }
+}
+
 db::DVector
 Service::snap (db::DVector v) const
 {
@@ -239,6 +358,13 @@ Service::configure (const std::string &name, const std::string &value)
 
     return true;  //  taken
 
+  } else if (name == cfg_edit_snap_objects_to_grid) {
+
+    tl::from_string (value, m_snap_objects_to_grid);
+    service_configuration_changed ();
+
+    return true;  //  taken
+
   } else if (name == cfg_edit_move_angle_mode) {
 
     acc.from_string (value, m_move_ac);
@@ -434,9 +560,14 @@ void
 Service::move (const db::DPoint &pu, lay::angle_constraint_type ac)
 {
   m_alt_ac = ac;
-  db::DPoint p = snap (m_move_start) + snap (pu - m_move_start, false /*move*/);
   if (view ()->is_editable () && m_moving) {
-    move_markers (db::DTrans (p - db::DPoint ()) * db::DTrans (m_move_trans.fp_trans ()) * db::DTrans (db::DPoint () - snap (m_move_start)));
+    db::DPoint ref = snap (m_move_start);
+    bool snapped = false;
+    db::DPoint p = ref + snap_marker_to_grid (pu - m_move_start, snapped);
+    if (! snapped) {
+      p = ref + snap (pu - m_move_start, false /*move*/);
+    }
+    move_markers (db::DTrans (p - db::DPoint ()) * db::DTrans (m_move_trans.fp_trans ()) * db::DTrans (db::DPoint () - ref));
   }
   m_alt_ac = lay::AC_Global;
 }
@@ -445,9 +576,14 @@ void
 Service::move_transform (const db::DPoint &pu, db::DFTrans tr, lay::angle_constraint_type ac)
 {
   m_alt_ac = ac;
-  db::DPoint p = snap (m_move_start) + snap (pu - m_move_start, false);
   if (view ()->is_editable () && m_moving) {
-    move_markers (db::DTrans (p - db::DPoint ()) * db::DTrans (m_move_trans.fp_trans () * tr) * db::DTrans (db::DPoint () - snap (m_move_start)));
+    db::DPoint ref = snap (m_move_start);
+    bool snapped = false;
+    db::DPoint p = ref + snap_marker_to_grid (pu - m_move_start, snapped);
+    if (! snapped) {
+      p = ref + snap (pu - m_move_start, false /*move*/);
+    }
+    move_markers (db::DTrans (p - db::DPoint ()) * db::DTrans (tr * m_move_trans.fp_trans ()) * db::DTrans (db::DPoint () - ref));
   }
   m_alt_ac = lay::AC_Global;
 }
@@ -471,6 +607,9 @@ Service::selection_bbox ()
   //  build the transformation variants cache 
   //  TODO: this is done multiple times - once for each service!
   TransformationVariants tv (view ());
+  const db::DCplxTrans &vp = view ()->viewport ().trans ();
+
+  lay::TextInfo text_info (view ());
 
   db::DBox box;
   for (objects::const_iterator r = m_selection.begin (); r != m_selection.end (); ++r) {
@@ -486,7 +625,13 @@ Service::selection_bbox ()
       const std::vector<db::DCplxTrans> *tv_list = tv.per_cv_and_layer (r->cv_index (), r->layer ());
       if (tv_list != 0) {
         for (std::vector<db::DCplxTrans>::const_iterator t = tv_list->begin (); t != tv_list->end (); ++t) {
-          box += *t * (ctx_trans * r->shape ().bbox ());
+          if (r->shape ().is_text ()) {
+            db::Text text;
+            r->shape ().text (text);
+            box += *t * text_info.bbox (ctx_trans * text, vp * *t);
+          } else {
+            box += *t * (ctx_trans * r->shape ().bbox ());
+          }
         }
       }
 
@@ -1100,7 +1245,20 @@ Service::transient_select (const db::DPoint &pos)
 
       lay::ShapeMarker *marker = new lay::ShapeMarker (view (), r->cv_index ());
       marker->set (r->shape (), gt, mp_view->cv_transform_variants (r->cv_index (), r->layer ()));
-      marker->set_vertex_size (0);
+
+      bool is_point = false;
+      if (r->shape ().is_edge () || r->shape ().is_box ()) {
+        is_point = r->shape ().bbox ().is_point ();
+      } else if (r->shape ().is_point ()) {
+        is_point = true;
+      }
+
+      if (is_point) {
+        marker->set_vertex_shape (lay::ViewOp::Cross);
+        marker->set_vertex_size (9 /*cross vertex size*/);
+      } else {
+        marker->set_vertex_size (0);
+      }
       marker->set_line_width (1);
       marker->set_halo (0);
 
@@ -1594,7 +1752,15 @@ Service::do_selection_to_view ()
         } 
 
         marker->set (r->shape (), gt, *tv_list);
-        if (r->shape ().is_text ()) {
+
+        bool is_point = false;
+        if (r->shape ().is_text () || r->shape ().is_point ()) {
+          is_point = true;
+        } else if (r->shape ().is_edge () || r->shape ().is_box ()) {
+          is_point = r->shape ().bbox ().is_point ();
+        }
+
+        if (is_point) {
           //  show the origins as crosses for texts
           marker->set_vertex_shape (lay::ViewOp::Cross);
           marker->set_vertex_size (9 /*cross vertex size*/);
