@@ -693,6 +693,16 @@ static Recipe_Impl *make_recipe (const std::string &name, const std::string &des
   return new Recipe_Impl (name, description);
 }
 
+static tl::Variant make_impl (const std::string &generator, const std::map<std::string, tl::Variant> &add_params)
+{
+  return Recipe_Impl::make (generator, add_params);
+}
+
+std::string generator_impl (Recipe_Impl *recipe, const std::map<std::string, tl::Variant> &params)
+{
+  return recipe->generator (params);
+}
+
 Class<Recipe_Impl> decl_Recipe_Impl ("tl", "Recipe",
   gsi::constructor ("new", &make_recipe, gsi::arg ("name"), gsi::arg ("description", std::string (), "\"\""),
     "@brief Creates a new recipe object with the given name and (optional) description"
@@ -703,13 +713,13 @@ Class<Recipe_Impl> decl_Recipe_Impl ("tl", "Recipe",
   gsi::method ("description", &Recipe_Impl::description,
     "@brief Gets the description of the recipe."
   ) +
-  gsi::method ("make", &Recipe_Impl::make, gsi::arg ("generator"), gsi::arg ("add_params", std::map<std::string, tl::Variant> (), "{}"),
+  gsi::method ("make", &make_impl, gsi::arg ("generator"), gsi::arg ("add_params", std::map<std::string, tl::Variant> (), "{}"),
     "@brief Executes the recipe given by the generator string.\n"
     "The generator string is the one delivered with \\generator.\n"
     "Additional parameters can be passed in \"add_params\". They have lower priority than the parameters "
     "kept inside the generator string."
   ) +
-  gsi::method ("generator", &Recipe_Impl::generator, gsi::arg ("params"),
+  gsi::method_ext ("generator", &generator_impl, gsi::arg ("params"),
     "@brief Delivers the generator string from the given parameters.\n"
     "The generator string can be used with \\make to re-run the recipe."
   ) +
