@@ -426,6 +426,22 @@ public:
     return m_synchronous;
   }
 
+  /**
+   *  @brief Sets the title string
+   *
+   *  If empty, the default title will be created. Otherwise this string will
+   *  be used. It is subject to expression interpolation.
+   */
+  void set_title (const std::string &title);
+
+  /**
+   *  @brief Gets the title string
+   */
+  const std::string &title () const
+  {
+    return m_title;
+  }
+
   /**
    *  @brief Returns true, if the edit functions of the current view are enabled
    */
@@ -681,6 +697,7 @@ protected slots:
   void message_timer ();
   void edits_enabled_changed ();
   void menu_needs_update ();
+  void technology_changed ();
 
   void file_changed_timer ();
   void file_changed (const QString &path);
@@ -689,6 +706,7 @@ protected slots:
 protected:
   void update_content ();
   void do_update_menu ();
+  void do_update_grids ();
   void do_update_mru_menus ();
   bool eventFilter (QObject *watched, QEvent *event);
 
@@ -736,6 +754,7 @@ private:
   bool m_disable_tab_selected;
   bool m_exited;
   tl::DeferredMethod<MainWindow> dm_do_update_menu;
+  tl::DeferredMethod<MainWindow> dm_do_update_grids;
   tl::DeferredMethod<MainWindow> dm_do_update_mru_menus;
   tl::DeferredMethod<MainWindow> dm_exit;
   QTimer m_message_timer;
@@ -748,6 +767,7 @@ private:
   std::string m_initial_technology;
   double m_grid_micron;
   std::vector<double> m_default_grids;
+  double m_default_grid;
   bool m_default_grids_updated;
   std::vector<std::pair<std::string, std::string> > m_key_bindings;
   bool m_new_layout_current_panel;
@@ -760,6 +780,7 @@ private:
   std::string m_message;
   std::unique_ptr<QPrinter> mp_printer;
   std::vector<QString> m_changed_files;
+  std::string m_title;
 
   //  the object manager (undo/redo mechanism and others)
   db::Manager m_manager;
