@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -320,6 +320,11 @@ public:
     return m_line_styles;
   }
 
+  /**
+   *  @brief Reimplementation of ViewObjectCanvas: Resolution
+   */
+  double resolution () const;
+
   /**
    *  @brief Reimplementation of ViewObjectCanvas: Background color 
    */
@@ -444,8 +449,6 @@ private:
   void do_redraw_all (bool force_redraw = true);
 
   void prepare_drawing ();
-  virtual double resolution () const;
-
   const std::vector<ViewOp> &scaled_view_ops (unsigned int lw);
 };
 
