@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -41,8 +41,8 @@ public:
   MoveService (lay::LayoutViewBase *view);
   ~MoveService ();
 
-  virtual bool configure (const std::string &name, const std::string &value);
-  bool begin_move (db::Transaction *transaction = 0, bool selected_after_move = true);
+  bool configure (const std::string &name, const std::string &value);
+  bool begin_move (db::Transaction *transaction = 0, bool transient_selection = false);
 
 private:
   virtual bool mouse_press_event (const db::DPoint &p, unsigned int buttons, bool prio);
