@@ -147,6 +147,17 @@ public:
   virtual bool drop_event (const db::DPoint & /*p*/, const DragDropDataBase * /*data*/) { return false; }
 #endif
 
+  /**
+   *  @brief Hover reset request
+   *
+   *  This event is issued for services providing some "hover" mode - i.e. capture
+   *  mouse move events and start a timer on them.
+   *
+   *  The implementation of this event should cancel this timer and
+   *  not raise a hover condition.
+   */
+  virtual void hover_reset () { }
+
   /**
    *  @brief Mouse press event handler
    *
@@ -605,6 +616,11 @@ public:
    */
   void drag_cancel ();
 
+  /**
+   *  @brief Calls hover_reset on all services
+   */
+  void hover_reset ();
+
   /**
    *  @brief CanvasPlane rendering 
    *
