@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -77,6 +77,9 @@ static const std::string cfg_sel_inside_pcells_mode ("sel-inside-pcells-mode");
 
 static const std::string cfg_tracking_cursor_color ("tracking-cursor-color");
 static const std::string cfg_tracking_cursor_enabled ("tracking-cursor-enabled");
+static const std::string cfg_crosshair_cursor_color ("crosshair-cursor-color");
+static const std::string cfg_crosshair_cursor_line_style ("crosshair-cursor-line-style");
+static const std::string cfg_crosshair_cursor_enabled ("crosshair-cursor-enabled");
 
 static const std::string cfg_markers_visible ("markers-visible");
 
@@ -95,6 +98,7 @@ static const std::string cfg_global_trans ("global-trans");
 static const std::string cfg_no_stipple ("no-stipple");
 static const std::string cfg_stipple_offset ("stipple-offset");
 static const std::string cfg_default_text_size ("default-text-size");
+static const std::string cfg_text_point_mode ("text-point-mode");
 static const std::string cfg_text_font ("text-font");
 static const std::string cfg_full_hier_new_cell ("full-hierarchy-new-cell");
 static const std::string cfg_initial_hier_depth ("initial-hier-depth");
@@ -113,6 +117,7 @@ static const std::string cfg_drop_small_cells_value ("drop-small-cells-value");
 static const std::string cfg_array_border_instances ("draw-array-border-instances");
 static const std::string cfg_default_lyp_file ("default-layer-properties");
 static const std::string cfg_default_add_other_layers ("default-add-other-layers");
+static const std::string cfg_layer_visibility_follows_selection ("layer-visibility-follows-selection");
 static const std::string cfg_layers_always_show_source ("layers-always-show-source");
 static const std::string cfg_layers_always_show_ld ("layers-always-show-ld");
 static const std::string cfg_layers_always_show_layout_index ("layers-always-show-layout-index");
