@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -202,9 +202,9 @@ BrowseShapesConfigPage::setup (lay::Dispatcher *root)
   cbx_window->setCurrentIndex (int (wmode));
 
   //  window dimension
-  double wdim = 1.0;
-  root->config_get (cfg_shb_window_dim, wdim);
-  le_window->setText (tl::to_qstring (tl::to_string (wdim)));
+  std::string wdim_str;
+  root->config_get (cfg_shb_window_dim, wdim_str);
+  mrg_window->set_margin (lay::Margin::from_string (wdim_str));
     
   //  max. instance count
   unsigned int max_inst_count = 1000;
@@ -230,15 +230,12 @@ BrowseShapesConfigPage::context_changed (int m)
 void
 BrowseShapesConfigPage::window_changed (int m)
 {
-  le_window->setEnabled (m == int (BrowseShapesForm::FitMarker) || m == int (BrowseShapesForm::CenterSize));
+  mrg_window->setEnabled (m == int (BrowseShapesForm::FitMarker) || m == int (BrowseShapesForm::CenterSize));
 }
 
 void 
 BrowseShapesConfigPage::commit (lay::Dispatcher *root)
 {
-  double dim = 1.0;
-  tl::from_string_ext (tl::to_string (le_window->text ()), dim);
-
   unsigned int max_inst_count = 1000;
   tl::from_string_ext (tl::to_string (le_max_inst->text ()), max_inst_count);
 
@@ -248,7 +245,7 @@ BrowseShapesConfigPage::commit (lay::Dispatcher *root)
   root->config_set (cfg_shb_context_cell, tl::to_string (le_cell_name->text ()));
   root->config_set (cfg_shb_context_mode, BrowseShapesForm::mode_type (cbx_context->currentIndex ()), BrowseShapesContextModeConverter ());
   root->config_set (cfg_shb_window_mode, BrowseShapesForm::window_type (cbx_window->currentIndex ()), BrowseShapesWindowModeConverter ());
-  root->config_set (cfg_shb_window_dim, dim);
+  root->config_set (cfg_shb_window_dim, mrg_window->get_margin ().to_string ());
   root->config_set (cfg_shb_max_inst_count, max_inst_count);
   root->config_set (cfg_shb_max_shape_count, max_shape_count);
 }
@@ -409,7 +406,7 @@ BrowseShapesForm::BrowseShapesForm (lay::Dispatcher *root, LayoutViewBase *vw)
     m_ef_enabled (true),
     m_mode (AnyTop),
     m_window (FitMarker),
-    m_window_dim (0.0),
+    m_window_dim (),
     m_max_inst_count (0), 
     m_max_shape_count (0)
 {
@@ -496,9 +493,8 @@ BrowseShapesForm::configure (const std::string &name, const std::string &value)
 
   } else if (name == cfg_shb_window_dim) {
 
-    double wdim = m_window_dim;
-    tl::from_string (value, wdim);
-    if (fabs (wdim - m_window_dim) > 1e-6) {
+    lay::Margin wdim = lay::Margin::from_string (value);
+    if (wdim != m_window_dim) {
       m_window_dim = wdim;
       need_update = true;
     }
@@ -961,16 +957,18 @@ BrowseShapesForm::highlight_current ()
 
   if (! dbox.empty ()) {
 
+    double window_dim = m_window_dim.get (dbox);
+
     view ()->select_cell (cell_index, m_cv_index);
     if (m_window == FitCell) {
       view ()->zoom_fit ();
     } else if (m_window == FitMarker) {
-      view ()->zoom_box (dbox.enlarged (db::DVector (m_window_dim, m_window_dim)));
+      view ()->zoom_box (dbox.enlarged (db::DVector (window_dim, window_dim)));
     } else if (m_window == Center) {
       view ()->pan_center (dbox.p1 () + (dbox.p2 () - dbox.p1 ()) * 0.5);
     } else if (m_window == CenterSize) {
-      double w = std::max (dbox.width (), m_window_dim);
-      double h = std::max (dbox.height (), m_window_dim);
+      double w = std::max (dbox.width (), window_dim);
+      double h = std::max (dbox.height (), window_dim);
       db::DPoint center (dbox.p1 () + (dbox.p2 () - dbox.p1 ()) * 0.5);
       db::DVector d (w * 0.5, h * 0.5);
       view ()->zoom_box (db::DBox (center - d, center + d));
